@@ -129,6 +129,8 @@ static GType gst_subbuffer_get_type (void);
 static GType _gst_subbuffer_type = 0;
 static GType _gst_buffer_type = 0;
 
+static GstMiniObjectClass *parent_class = NULL;
+
 void
 _gst_buffer_initialize (void)
 {
@@ -167,6 +169,8 @@ gst_buffer_class_init (gpointer g_class, gpointer class_data)
 {
   GstBufferClass *buffer_class = GST_BUFFER_CLASS (g_class);
 
+  parent_class = g_type_class_peek_parent (g_class);
+
   buffer_class->mini_object_class.copy =
       (GstMiniObjectCopyFunction) _gst_buffer_copy;
   buffer_class->mini_object_class.finalize =
@@ -182,9 +186,12 @@ gst_buffer_finalize (GstBuffer * buffer)
   GST_CAT_LOG (GST_CAT_BUFFER, "finalize %p", buffer);
 
   /* free our data */
-  g_free (buffer->malloc_data);
+  if (G_LIKELY (buffer->malloc_data))
+    buffer->free_func (buffer->malloc_data);
 
   gst_caps_replace (&GST_BUFFER_CAPS (buffer), NULL);
+
+  parent_class->finalize (GST_MINI_OBJECT_CAST (buffer));
 }
 
 /**
@@ -273,6 +280,7 @@ gst_buffer_init (GTypeInstance * instance, gpointer g_class)
   GST_BUFFER_DURATION (buffer) = GST_CLOCK_TIME_NONE;
   GST_BUFFER_OFFSET (buffer) = GST_BUFFER_OFFSET_NONE;
   GST_BUFFER_OFFSET_END (buffer) = GST_BUFFER_OFFSET_NONE;
+  GST_BUFFER_FREE_FUNC (buffer) = g_free;
 }
 
 /**
@@ -380,8 +388,9 @@ gst_buffer_try_new_and_alloc (guint size)
  * Returns: a reference to the #GstCaps. unref after usage.
  * Returns NULL if there were no caps on this buffer.
  */
-/* FIXME can we make this threadsafe without a lock on the buffer?
- * We can use compare and swap and atomic reads. */
+/* this is not made atomic because if the buffer were reffed from multiple
+ * threads, it would have a refcount > 2 and thus be immutable.
+ */
 GstCaps *
 gst_buffer_get_caps (GstBuffer * buffer)
 {
@@ -406,10 +415,9 @@ gst_buffer_get_caps (GstBuffer * buffer)
  * be increased and any previous caps on the buffer will be
  * unreffed.
  */
-/* FIXME can we make this threadsafe without a lock on the buffer?
- * We can use compare and swap and atomic reads. Another idea is to
- * not attach the caps to the buffer but use an event to signal a caps
- * change. */
+/* this is not made atomic because if the buffer were reffed from multiple
+ * threads, it would have a refcount > 2 and thus be immutable.
+ */
 void
 gst_buffer_set_caps (GstBuffer * buffer, GstCaps * caps)
 {
@@ -458,12 +466,6 @@ gst_buffer_make_metadata_writable (GstBuffer * buf)
   } else {
     ret = gst_buffer_create_sub (buf, 0, GST_BUFFER_SIZE (buf));
 
-    /* copy all the flags except IN_CAPS */
-    GST_BUFFER_FLAGS (ret) = GST_BUFFER_FLAGS (buf);
-    GST_BUFFER_FLAG_UNSET (ret, GST_BUFFER_FLAG_IN_CAPS);
-    /* data should always be set to READONLY */
-    GST_BUFFER_FLAG_SET (ret, GST_BUFFER_FLAG_READONLY);
-
     gst_buffer_unref (buf);
   }
 
@@ -593,6 +595,16 @@ gst_buffer_create_sub (GstBuffer * buffer, guint offset, guint size)
   GST_BUFFER_DATA (GST_BUFFER_CAST (subbuffer)) = buffer->data + offset;
   GST_BUFFER_SIZE (GST_BUFFER_CAST (subbuffer)) = size;
 
+  if ((offset == 0) && (size == GST_BUFFER_SIZE (buffer))) {
+    /* copy all the flags except IN_CAPS */
+    GST_BUFFER_FLAG_SET (subbuffer, GST_BUFFER_FLAGS (buffer));
+    GST_BUFFER_FLAG_UNSET (subbuffer, GST_BUFFER_FLAG_IN_CAPS);
+  } else {
+    /* copy only PREROLL & GAP flags */
+    GST_BUFFER_FLAG_SET (subbuffer, (GST_BUFFER_FLAGS (buffer) &
+            (GST_BUFFER_FLAG_PREROLL | GST_BUFFER_FLAG_GAP)));
+  }
+
   /* we can copy the timestamp and offset if the new buffer starts at
    * offset 0 */
   if (offset == 0) {
