@@ -48,7 +48,9 @@
  * bus to handle them.
  * Alternatively the application can register an asynchronous bus function
  * using gst_bus_add_watch_full() or gst_bus_add_watch(). This function will
- * receive messages a short while after they have been posted.
+ * install a #GSource in the default glib main loop and will deliver messages 
+ * a short while after they have been posted. Note that the main loop should 
+ * be running for the asynchronous callbacks.
  *
  * It is also possible to get messages from the bus without any thread
  * marshalling with the gst_bus_set_sync_handler() method. This makes it
@@ -61,16 +63,16 @@
  * Note that a #GstPipeline will set its bus into flushing state when changing
  * from READY to NULL state.
  *
- * Last reviewed on 2005-10-28 (0.9.4)
+ * Last reviewed on 2006-03-12 (0.10.5)
  */
 
+#include "gst_private.h"
 #include <errno.h>
 #ifdef HAVE_UNISTD_H
 #  include <unistd.h>
 #endif
 #include <sys/types.h>
 
-#include "gst_private.h"
 #include "gstinfo.h"
 
 #include "gstbus.h"
@@ -93,16 +95,20 @@ static void gst_bus_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_bus_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
+static void gst_bus_set_main_context (GstBus * bus, GMainContext * ctx);
 
 static GstObjectClass *parent_class = NULL;
 static guint gst_bus_signals[LAST_SIGNAL] = { 0 };
 
-/* the context we wakeup when we posted a message on the bus */
-static GMainContext *main_context;
-
 struct _GstBusPrivate
 {
   guint num_sync_message_emitters;
+
+  GCond *queue_cond;
+
+  GSource *watch_id;
+
+  GMainContext *main_context;
 };
 
 GType
@@ -151,8 +157,8 @@ marshal_VOID__MINIOBJECT (GClosure * closure, GValue * return_value,
     data2 = closure->data;
   }
   callback =
-      (marshalfunc_VOID__MINIOBJECT) (marshal_data ? marshal_data : cc->
-      callback);
+      (marshalfunc_VOID__MINIOBJECT) (marshal_data ? marshal_data :
+      cc->callback);
 
   callback (data1, gst_value_get_mini_object (param_values + 1), data2);
 }
@@ -181,7 +187,14 @@ gst_bus_class_init (GstBusClass * klass)
    * @message: the message that has been posted synchronously
    *
    * A message has been posted on the bus. This signal is emitted from the
-   * thread that posted the message so one has to be carefull with locking.
+   * thread that posted the message so one has to be careful with locking.
+   *
+   * This signal will not be emitted by default, you have to set up
+   * gst_bus_sync_signal_handler() as a sync handler if you want this
+   * signal to be emitted when a message is posted on the bus, like this:
+   * <programlisting>
+   * gst_bus_set_sync_handler (bus, gst_bus_sync_signal_handler, yourdata);
+   * </programlisting>
    */
   gst_bus_signals[SYNC_MESSAGE] =
       g_signal_new ("sync-message", G_TYPE_FROM_CLASS (klass),
@@ -195,7 +208,8 @@ gst_bus_class_init (GstBusClass * klass)
    * @message: the message that has been posted asynchronously
    *
    * A message has been posted on the bus. This signal is emitted from a
-   * GSource added to the mainloop.
+   * GSource added to the mainloop. this signal will only be emitted when
+   * there is a mainloop running.
    */
   gst_bus_signals[ASYNC_MESSAGE] =
       g_signal_new ("message", G_TYPE_FROM_CLASS (klass),
@@ -203,8 +217,6 @@ gst_bus_class_init (GstBusClass * klass)
       G_STRUCT_OFFSET (GstBusClass, message), NULL, NULL,
       marshal_VOID__MINIOBJECT, G_TYPE_NONE, 1, GST_TYPE_MESSAGE);
 
-  main_context = g_main_context_default ();
-
   g_type_class_add_private (klass, sizeof (GstBusPrivate));
 }
 
@@ -215,6 +227,7 @@ gst_bus_init (GstBus * bus)
   bus->queue_lock = g_mutex_new ();
 
   bus->priv = G_TYPE_INSTANCE_GET_PRIVATE (bus, GST_TYPE_BUS, GstBusPrivate);
+  bus->priv->queue_cond = g_cond_new ();
 
   GST_DEBUG_OBJECT (bus, "created");
 }
@@ -240,6 +253,13 @@ gst_bus_dispose (GObject * object)
     g_mutex_unlock (bus->queue_lock);
     g_mutex_free (bus->queue_lock);
     bus->queue_lock = NULL;
+    g_cond_free (bus->priv->queue_cond);
+    bus->priv->queue_cond = NULL;
+  }
+
+  if (bus->priv->main_context) {
+    g_main_context_unref (bus->priv->main_context);
+    bus->priv->main_context = NULL;
   }
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
@@ -275,6 +295,42 @@ gst_bus_get_property (GObject * object, guint prop_id,
   }
 }
 
+static void
+gst_bus_wakeup_main_context (GstBus * bus)
+{
+  GMainContext *ctx;
+
+  GST_OBJECT_LOCK (bus);
+  if ((ctx = bus->priv->main_context))
+    g_main_context_ref (ctx);
+  GST_OBJECT_UNLOCK (bus);
+
+  g_main_context_wakeup (ctx);
+
+  if (ctx)
+    g_main_context_unref (ctx);
+}
+
+static void
+gst_bus_set_main_context (GstBus * bus, GMainContext * ctx)
+{
+  GST_OBJECT_LOCK (bus);
+
+  if (bus->priv->main_context != NULL) {
+    g_main_context_unref (bus->priv->main_context);
+    bus->priv->main_context = NULL;
+  }
+
+  if (ctx != NULL) {
+    bus->priv->main_context = g_main_context_ref (ctx);
+  }
+
+  GST_DEBUG_OBJECT (bus, "setting main context to %p, GLib default context: %p",
+      ctx, g_main_context_default ());
+
+  GST_OBJECT_UNLOCK (bus);
+}
+
 /**
  * gst_bus_new:
  *
@@ -316,8 +372,10 @@ gst_bus_post (GstBus * bus, GstMessage * message)
   g_return_val_if_fail (GST_IS_BUS (bus), FALSE);
   g_return_val_if_fail (GST_IS_MESSAGE (message), FALSE);
 
-  GST_DEBUG_OBJECT (bus, "[msg %p] posting on bus, type %s",
-      message, GST_MESSAGE_TYPE_NAME (message));
+  GST_DEBUG_OBJECT (bus, "[msg %p] posting on bus, type %s, %" GST_PTR_FORMAT
+      " from source %" GST_PTR_FORMAT,
+      message, GST_MESSAGE_TYPE_NAME (message), message->structure,
+      message->src);
 
   GST_OBJECT_LOCK (bus);
   /* check if the bus is flushing */
@@ -350,11 +408,11 @@ gst_bus_post (GstBus * bus, GstMessage * message)
       GST_DEBUG_OBJECT (bus, "[msg %p] pushing on async queue", message);
       g_mutex_lock (bus->queue_lock);
       g_queue_push_tail (bus->queue, message);
+      g_cond_broadcast (bus->priv->queue_cond);
       g_mutex_unlock (bus->queue_lock);
       GST_DEBUG_OBJECT (bus, "[msg %p] pushed on async queue", message);
 
-      /* FIXME cannot assume sources are only in the default context */
-      g_main_context_wakeup (main_context);
+      gst_bus_wakeup_main_context (bus);
 
       break;
     case GST_BUS_ASYNC:
@@ -375,10 +433,10 @@ gst_bus_post (GstBus * bus, GstMessage * message)
       g_mutex_lock (lock);
       g_mutex_lock (bus->queue_lock);
       g_queue_push_tail (bus->queue, message);
+      g_cond_broadcast (bus->priv->queue_cond);
       g_mutex_unlock (bus->queue_lock);
 
-      /* FIXME cannot assume sources are only in the default context */
-      g_main_context_wakeup (main_context);
+      gst_bus_wakeup_main_context (bus);
 
       /* now block till the message is freed */
       g_cond_wait (cond, lock);
@@ -467,6 +525,154 @@ gst_bus_set_flushing (GstBus * bus, gboolean flushing)
   GST_OBJECT_UNLOCK (bus);
 }
 
+/**
+ * gst_bus_timed_pop_filtered:
+ * @bus: a #GstBus to pop from
+ * @timeout: a timeout in nanoseconds, or GST_CLOCK_TIME_NONE to wait forever
+ * @types: message types to take into account, GST_MESSAGE_ANY for any type
+ *
+ * Get a message from the bus whose type matches the message type mask @types,
+ * waiting up to the specified timeout (and discarding any messages that do not
+ * match the mask provided).
+ *
+ * If @timeout is 0, this function behaves like gst_bus_pop_filtered(). If
+ * @timeout is #GST_CLOCK_TIME_NONE, this function will block forever until a
+ * matching message was posted on the bus.
+ *
+ * Returns: a #GstMessage matching the filter in @types, or NULL if no matching
+ * message was found on the bus until the timeout expired.
+ * The message is taken from the bus and needs to be unreffed with
+ * gst_message_unref() after usage.
+ *
+ * MT safe.
+ *
+ * Since: 0.10.15
+ */
+GstMessage *
+gst_bus_timed_pop_filtered (GstBus * bus, GstClockTime timeout,
+    GstMessageType types)
+{
+  GstMessage *message;
+  GTimeVal *timeval, abstimeout;
+  gboolean first_round = TRUE;
+
+  g_return_val_if_fail (GST_IS_BUS (bus), NULL);
+  g_return_val_if_fail (types != 0, NULL);
+
+  g_mutex_lock (bus->queue_lock);
+
+  while (TRUE) {
+    GST_LOG_OBJECT (bus, "have %d messages", g_queue_get_length (bus->queue));
+
+    while ((message = g_queue_pop_head (bus->queue))) {
+      GST_DEBUG_OBJECT (bus, "got message %p, %s, type mask is %u",
+          message, GST_MESSAGE_TYPE_NAME (message), (guint) types);
+      if ((GST_MESSAGE_TYPE (message) & types) != 0) {
+        /* exit the loop, we have a message */
+        goto beach;
+      } else {
+        GST_DEBUG_OBJECT (bus, "discarding message, does not match mask");
+        gst_message_unref (message);
+        message = NULL;
+      }
+    }
+
+    /* no need to wait, exit loop */
+    if (timeout == 0)
+      break;
+
+    if (timeout == GST_CLOCK_TIME_NONE) {
+      /* wait forever */
+      timeval = NULL;
+    } else if (first_round) {
+      glong add = timeout / 1000;
+
+      if (add == 0)
+        /* no need to wait */
+        break;
+
+      /* make timeout absolute */
+      g_get_current_time (&abstimeout);
+      g_time_val_add (&abstimeout, add);
+      timeval = &abstimeout;
+      first_round = FALSE;
+      GST_DEBUG_OBJECT (bus, "blocking for message, timeout %ld", add);
+    } else {
+      /* calculated the absolute end time already, no need to do it again */
+      GST_DEBUG_OBJECT (bus, "blocking for message, again");
+      timeval = &abstimeout;    /* fool compiler */
+    }
+    if (!g_cond_timed_wait (bus->priv->queue_cond, bus->queue_lock, timeval)) {
+      GST_INFO_OBJECT (bus, "timed out, breaking loop");
+      break;
+    } else {
+      GST_INFO_OBJECT (bus, "we got woken up, recheck for message");
+    }
+  }
+
+beach:
+
+  g_mutex_unlock (bus->queue_lock);
+
+  return message;
+}
+
+
+/**
+ * gst_bus_timed_pop:
+ * @bus: a #GstBus to pop
+ * @timeout: a timeout
+ *
+ * Get a message from the bus, waiting up to the specified timeout.
+ *
+ * If @timeout is 0, this function behaves like gst_bus_pop(). If @timeout is
+ * #GST_CLOCK_TIME_NONE, this function will block forever until a message was
+ * posted on the bus.
+ *
+ * Returns: The #GstMessage that is on the bus after the specified timeout
+ * or NULL if the bus is empty after the timeout expired.
+ * The message is taken from the bus and needs to be unreffed with
+ * gst_message_unref() after usage.
+ *
+ * MT safe.
+ *
+ * Since: 0.10.12
+ */
+GstMessage *
+gst_bus_timed_pop (GstBus * bus, GstClockTime timeout)
+{
+  g_return_val_if_fail (GST_IS_BUS (bus), NULL);
+
+  return gst_bus_timed_pop_filtered (bus, timeout, GST_MESSAGE_ANY);
+}
+
+/**
+ * gst_bus_pop_filtered:
+ * @bus: a #GstBus to pop
+ * @types: message types to take into account
+ *
+ * Get a message matching @type from the bus.  Will discard all messages on
+ * the bus that do not match @type and that have been posted before the first
+ * message that does match @type.  If there is no message matching @type on
+ * the bus, all messages will be discarded.
+ *
+ * Returns: The next #GstMessage matching @type that is on the bus, or NULL if
+ *     the bus is empty or there is no message matching @type.
+ * The message is taken from the bus and needs to be unreffed with
+ * gst_message_unref() after usage.
+ *
+ * MT safe.
+ *
+ * Since: 0.10.15
+ */
+GstMessage *
+gst_bus_pop_filtered (GstBus * bus, GstMessageType types)
+{
+  g_return_val_if_fail (GST_IS_BUS (bus), NULL);
+  g_return_val_if_fail (types != 0, NULL);
+
+  return gst_bus_timed_pop_filtered (bus, 0, types);
+}
 
 /**
  * gst_bus_pop:
@@ -475,27 +681,17 @@ gst_bus_set_flushing (GstBus * bus, gboolean flushing)
  * Get a message from the bus.
  *
  * Returns: The #GstMessage that is on the bus, or NULL if the bus is empty.
+ * The message is taken from the bus and needs to be unreffed with
+ * gst_message_unref() after usage.
  *
  * MT safe.
  */
 GstMessage *
 gst_bus_pop (GstBus * bus)
 {
-  GstMessage *message;
-
   g_return_val_if_fail (GST_IS_BUS (bus), NULL);
 
-  g_mutex_lock (bus->queue_lock);
-  message = g_queue_pop_head (bus->queue);
-  if (message)
-    GST_DEBUG_OBJECT (bus, "pop from bus, have %d messages, got message %p, %s",
-        g_queue_get_length (bus->queue) + 1, message,
-        GST_MESSAGE_TYPE_NAME (message));
-  else
-    GST_DEBUG_OBJECT (bus, "pop from bus, no messages");
-  g_mutex_unlock (bus->queue_lock);
-
-  return message;
+  return gst_bus_timed_pop_filtered (bus, 0, GST_MESSAGE_ANY);
 }
 
 /**
@@ -576,6 +772,7 @@ typedef struct
 {
   GSource source;
   GstBus *bus;
+  gboolean inited;
 } GstBusSource;
 
 static gboolean
@@ -583,6 +780,14 @@ gst_bus_source_prepare (GSource * source, gint * timeout)
 {
   GstBusSource *bsrc = (GstBusSource *) source;
 
+  /* we do this here now that we know that we're attached to a main context
+   * (we don't support detaching a source from a main context and then
+   * re-attaching it to a different main context) */
+  if (G_UNLIKELY (!bsrc->inited)) {
+    gst_bus_set_main_context (bsrc->bus, g_source_get_context (source));
+    bsrc->inited = TRUE;
+  }
+
   *timeout = -1;
   return gst_bus_have_pending (bsrc->bus);
 }
@@ -612,7 +817,11 @@ gst_bus_source_dispatch (GSource * source, GSourceFunc callback,
   g_return_val_if_fail (GST_IS_BUS (bus), FALSE);
 
   message = gst_bus_pop (bus);
-  g_return_val_if_fail (message != NULL, FALSE);
+
+  /* The message queue might be empty if some other thread or callback set
+   * the bus to flushing between check/prepare and dispatch */
+  if (G_UNLIKELY (message == NULL))
+    return TRUE;
 
   if (!handler)
     goto no_handler;
@@ -629,7 +838,7 @@ gst_bus_source_dispatch (GSource * source, GSourceFunc callback,
 no_handler:
   {
     g_warning ("GstBus watch dispatched without callback\n"
-        "You must call g_source_connect().");
+        "You must call g_source_set_callback().");
     gst_message_unref (message);
     return FALSE;
   }
@@ -639,7 +848,18 @@ static void
 gst_bus_source_finalize (GSource * source)
 {
   GstBusSource *bsource = (GstBusSource *) source;
+  GstBus *bus;
 
+  bus = bsource->bus;
+
+  GST_DEBUG_OBJECT (bus, "finalize source %p", source);
+
+  GST_OBJECT_LOCK (bus);
+  if (bus->priv->watch_id == source)
+    bus->priv->watch_id = NULL;
+  GST_OBJECT_UNLOCK (bus);
+
+  gst_bus_set_main_context (bsource->bus, NULL);
   gst_object_unref (bsource->bus);
   bsource->bus = NULL;
 }
@@ -670,40 +890,25 @@ gst_bus_create_watch (GstBus * bus)
 
   source = (GstBusSource *) g_source_new (&gst_bus_source_funcs,
       sizeof (GstBusSource));
-  gst_object_ref (bus);
-  source->bus = bus;
+  source->bus = gst_object_ref (bus);
+  source->inited = FALSE;
 
   return (GSource *) source;
 }
 
-/**
- * gst_bus_add_watch_full:
- * @bus: a #GstBus to create the watch for.
- * @priority: The priority of the watch.
- * @func: A function to call when a message is received.
- * @user_data: user data passed to @func.
- * @notify: the function to call when the source is removed.
- *
- * Adds a bus watch to the default main context with the given priority.
- * If the func returns FALSE, the source will be removed.
- *
- * When the func is called, the message belongs to the caller; if you want to
- * keep a copy of it, call gst_message_ref() before leaving the func.
- *
- * The watch can be removed using #g_source_remove().
- *
- * Returns: The event source id.
- *
- * MT safe.
- */
-guint
-gst_bus_add_watch_full (GstBus * bus, gint priority,
+/* must be called with the bus OBJECT LOCK */
+static guint
+gst_bus_add_watch_full_unlocked (GstBus * bus, gint priority,
     GstBusFunc func, gpointer user_data, GDestroyNotify notify)
 {
   guint id;
   GSource *source;
 
-  g_return_val_if_fail (GST_IS_BUS (bus), 0);
+  if (bus->priv->watch_id) {
+    GST_ERROR_OBJECT (bus,
+        "Tried to add new watch while one was already there");
+    return 0;
+  }
 
   source = gst_bus_create_watch (bus);
 
@@ -715,7 +920,49 @@ gst_bus_add_watch_full (GstBus * bus, gint priority,
   id = g_source_attach (source, NULL);
   g_source_unref (source);
 
-  GST_DEBUG_OBJECT (bus, "New source %p", source);
+  if (id) {
+    bus->priv->watch_id = source;
+  }
+
+  GST_DEBUG_OBJECT (bus, "New source %p with id %u", source, id);
+  return id;
+}
+
+/**
+ * gst_bus_add_watch_full:
+ * @bus: a #GstBus to create the watch for.
+ * @priority: The priority of the watch.
+ * @func: A function to call when a message is received.
+ * @user_data: user data passed to @func.
+ * @notify: the function to call when the source is removed.
+ *
+ * Adds a bus watch to the default main context with the given @priority.
+ * This function is used to receive asynchronous messages in the main loop.
+ * There can only be a single bus watch per bus, you must remove it before you
+ * can set a new one.
+ *
+ * When @func is called, the message belongs to the caller; if you want to
+ * keep a copy of it, call gst_message_ref() before leaving @func.
+ *
+ * The watch can be removed using g_source_remove() or by returning FALSE
+ * from @func.
+ *
+ * Returns: The event source id.
+ *
+ * MT safe.
+ */
+guint
+gst_bus_add_watch_full (GstBus * bus, gint priority,
+    GstBusFunc func, gpointer user_data, GDestroyNotify notify)
+{
+  guint id;
+
+  g_return_val_if_fail (GST_IS_BUS (bus), 0);
+
+  GST_OBJECT_LOCK (bus);
+  id = gst_bus_add_watch_full_unlocked (bus, priority, func, user_data, notify);
+  GST_OBJECT_UNLOCK (bus);
+
   return id;
 }
 
@@ -726,8 +973,12 @@ gst_bus_add_watch_full (GstBus * bus, gint priority,
  * @user_data: user data passed to @func.
  *
  * Adds a bus watch to the default main context with the default priority.
+ * This function is used to receive asynchronous messages in the main loop.
+ * There can only be a single bus watch per bus, you must remove it before you
+ * can set a new one.
  *
- * The watch can be removed using #g_source_remove().
+ * The watch can be removed using g_source_remove() or by returning FALSE
+ * from @func.
  *
  * Returns: The event source id.
  *
@@ -752,17 +1003,23 @@ typedef struct
 static void
 poll_func (GstBus * bus, GstMessage * message, GstBusPollData * poll_data)
 {
+  GstMessageType type;
+
   if (!g_main_loop_is_running (poll_data->loop)) {
     GST_DEBUG ("mainloop %p not running", poll_data->loop);
     return;
   }
 
-  if (GST_MESSAGE_TYPE (message) & poll_data->events) {
+  type = GST_MESSAGE_TYPE (message);
+
+  if (type & poll_data->events) {
     g_return_if_fail (poll_data->message == NULL);
     /* keep ref to message */
     poll_data->message = gst_message_ref (message);
     GST_DEBUG ("mainloop %p quit", poll_data->loop);
     g_main_loop_quit (poll_data->loop);
+  } else {
+    GST_DEBUG ("type %08x does not match %08x", type, poll_data->events);
   }
 }
 
@@ -802,7 +1059,8 @@ poll_destroy_timeout (GstBusPollData * poll_data)
  * @bus: a #GstBus
  * @events: a mask of #GstMessageType, representing the set of message types to
  * poll for.
- * @timeout: the poll timeout, as a #GstClockTimeDiff, or -1 to poll indefinitely.
+ * @timeout: the poll timeout, as a #GstClockTimeDiff, or -1 to poll
+ * indefinitely.
  *
  * Poll the bus for messages. Will block while waiting for messages to come.
  * You can specify a maximum time to poll with the @timeout parameter. If
@@ -819,8 +1077,26 @@ poll_destroy_timeout (GstBusPollData * poll_data)
  * This function will run a main loop from the default main context when
  * polling.
  *
+ * You should never use this function, since it is pure evil. This is
+ * especially true for GUI applications based on Gtk+ or Qt, but also for any
+ * other non-trivial application that uses the GLib main loop. As this function
+ * runs a GLib main loop, any callback attached to the default GLib main
+ * context may be invoked. This could be timeouts, GUI events, I/O events etc.;
+ * even if gst_bus_poll() is called with a 0 timeout. Any of these callbacks
+ * may do things you do not expect, e.g. destroy the main application window or
+ * some other resource; change other application state; display a dialog and
+ * run another main loop until the user clicks it away. In short, using this
+ * function may add a lot of complexity to your code through unexpected
+ * re-entrancy and unexpected changes to your application's state.
+ *
+ * For 0 timeouts use gst_bus_pop_filtered() instead of this function; for
+ * other short timeouts use gst_bus_timed_pop_filtered(); everything else is
+ * better handled by setting up an asynchronous bus watch and doing things
+ * from there.
+ *
  * Returns: The message that was received, or NULL if the poll timed out.
- * The message is taken from the bus and needs to be unreffed after usage.
+ * The message is taken from the bus and needs to be unreffed with
+ * gst_message_unref() after usage.
  */
 GstMessage *
 gst_bus_poll (GstBus * bus, GstMessageType events, GstClockTimeDiff timeout)
@@ -874,7 +1150,7 @@ gst_bus_poll (GstBus * bus, GstMessageType events, GstClockTimeDiff timeout)
  * @message: the #GstMessage received
  * @data: user data
  *
- * A helper GstBusFunc that can be used to convert all asynchronous messages
+ * A helper #GstBusFunc that can be used to convert all asynchronous messages
  * into signals.
  *
  * Returns: TRUE
@@ -925,7 +1201,7 @@ gst_bus_sync_signal_handler (GstBus * bus, GstMessage * message, gpointer data)
  * gst_bus_enable_sync_message_emission:
  * @bus: a #GstBus on which you want to receive the "sync-message" signal
  *
- * Instructs GStreamer to emit the sync-message signal after running the bus's
+ * Instructs GStreamer to emit the "sync-message" signal after running the bus's
  * sync handler. This function is here so that code can ensure that they can
  * synchronously receive messages without having to affect what the bin's sync
  * handler is. 
@@ -936,9 +1212,9 @@ gst_bus_sync_signal_handler (GstBus * bus, GstMessage * message, gpointer data)
  *
  * While this function looks similar to gst_bus_add_signal_watch(), it is not
  * exactly the same -- this function enables <emphasis>synchronous</emphasis> emission of
- * signals when messages arrive; gst_bus_add_signal_watch adds an idle callback
+ * signals when messages arrive; gst_bus_add_signal_watch() adds an idle callback
  * to pop messages off the bus <emphasis>asynchronously</emphasis>. The sync-message signal
- * comes from the thread of whatever object posted the message; the message
+ * comes from the thread of whatever object posted the message; the "message"
  * signal is marshalled to the main thread via the main loop.
  *
  * MT safe.
@@ -949,9 +1225,7 @@ gst_bus_enable_sync_message_emission (GstBus * bus)
   g_return_if_fail (GST_IS_BUS (bus));
 
   GST_OBJECT_LOCK (bus);
-
   bus->priv->num_sync_message_emitters++;
-
   GST_OBJECT_UNLOCK (bus);
 }
 
@@ -960,14 +1234,14 @@ gst_bus_enable_sync_message_emission (GstBus * bus)
  * @bus: a #GstBus on which you previously called
  * gst_bus_enable_sync_message_emission()
  *
- * Instructs GStreamer to stop emitting the sync-message signal for this bus.
+ * Instructs GStreamer to stop emitting the "sync-message" signal for this bus.
  * See gst_bus_enable_sync_message_emission() for more information.
  *
  * In the event that multiple pieces of code have called
  * gst_bus_enable_sync_message_emission(), the sync-message emissions will only
  * be stopped after all calls to gst_bus_enable_sync_message_emission() were
  * "cancelled" by calling this function. In this way the semantics are exactly
- * the same as gst_object_ref(); that which calls enable should also call
+ * the same as gst_object_ref() that which calls enable should also call
  * disable.
  *
  * MT safe.
@@ -976,13 +1250,10 @@ void
 gst_bus_disable_sync_message_emission (GstBus * bus)
 {
   g_return_if_fail (GST_IS_BUS (bus));
-
   g_return_if_fail (bus->num_signal_watchers == 0);
 
   GST_OBJECT_LOCK (bus);
-
   bus->priv->num_sync_message_emitters--;
-
   GST_OBJECT_UNLOCK (bus);
 }
 
@@ -992,13 +1263,16 @@ gst_bus_disable_sync_message_emission (GstBus * bus)
  * @priority: The priority of the watch.
  *
  * Adds a bus signal watch to the default main context with the given priority.
- * After calling this statement, the bus will emit the message signal for each
- * message posted on the bus.
+ * After calling this statement, the bus will emit the "message" signal for each
+ * message posted on the bus when the main loop is running.
  *
  * This function may be called multiple times. To clean up, the caller is
  * responsible for calling gst_bus_remove_signal_watch() as many times as this
  * function is called.
  *
+ * There can only be a single bus watch per bus, you most remove all signal watch
+ * before you can set another type of watch.
+ *
  * MT safe.
  */
 void
@@ -1012,17 +1286,30 @@ gst_bus_add_signal_watch_full (GstBus * bus, gint priority)
   if (bus->num_signal_watchers > 0)
     goto done;
 
+  /* this should not fail because the counter above takes care of it */
   g_assert (bus->signal_watch_id == 0);
 
   bus->signal_watch_id =
-      gst_bus_add_watch_full (bus, priority, gst_bus_async_signal_func, NULL,
-      NULL);
+      gst_bus_add_watch_full_unlocked (bus, priority, gst_bus_async_signal_func,
+      NULL, NULL);
+
+  if (G_UNLIKELY (bus->signal_watch_id == 0))
+    goto add_failed;
 
 done:
 
   bus->num_signal_watchers++;
 
   GST_OBJECT_UNLOCK (bus);
+  return;
+
+  /* ERRORS */
+add_failed:
+  {
+    g_critical ("Could not add signal watch to bus %s", GST_OBJECT_NAME (bus));
+    GST_OBJECT_UNLOCK (bus);
+    return;
+  }
 }
 
 /**
@@ -1031,7 +1318,7 @@ done:
  *
  * Adds a bus signal watch to the default main context with the default
  * priority.
- * After calling this statement, the bus will emit the message signal for each
+ * After calling this statement, the bus will emit the "message" signal for each
  * message posted on the bus.
  *
  * This function may be called multiple times. To clean up, the caller is
@@ -1057,6 +1344,8 @@ gst_bus_add_signal_watch (GstBus * bus)
 void
 gst_bus_remove_signal_watch (GstBus * bus)
 {
+  guint id = 0;
+
   g_return_if_fail (GST_IS_BUS (bus));
 
   /* I know the callees don't take this lock, so go ahead and abuse it */
@@ -1070,13 +1359,20 @@ gst_bus_remove_signal_watch (GstBus * bus)
   if (bus->num_signal_watchers > 0)
     goto done;
 
-  g_source_remove (bus->signal_watch_id);
+  id = bus->signal_watch_id;
   bus->signal_watch_id = 0;
 
+  GST_DEBUG_OBJECT (bus, "removing signal watch %u", id);
+
 done:
   GST_OBJECT_UNLOCK (bus);
+
+  if (id)
+    g_source_remove (id);
+
   return;
 
+  /* ERRORS */
 error:
   {
     g_critical ("Bus %s has no signal watches attached", GST_OBJECT_NAME (bus));
