@@ -26,11 +26,11 @@
  * They are composed of an array of #GstStructure.
  *
  * Caps are exposed on #GstPadTemplate to describe all possible types a
- * given pad can handle. They are also stored in the registry along with
- * a description of the element.
+ * given pad can handle. They are also stored in the #GstRegistry along with
+ * a description of the #GstElement.
  *
- * Caps are exposed on the element pads using the gst_pad_get_caps() pad 
- * function. This function describes the possible types that the pad can 
+ * Caps are exposed on the element pads using the gst_pad_get_caps() pad
+ * function. This function describes the possible types that the pad can
  * handle or produce at runtime.
  *
  * Caps are also attached to buffers to describe to content of the data
@@ -45,10 +45,10 @@
  *  GstCaps *caps;
  *  caps = gst_caps_new_simple ("video/x-raw-yuv",
  *       "format", GST_TYPE_FOURCC, GST_MAKE_FOURCC ('I', '4', '2', '0'),
- *       "framerate", G_TYPE_DOUBLE, 25.0,
+ *       "framerate", GST_TYPE_FRACTION, 25, 1,
  *       "pixel-aspect-ratio", GST_TYPE_FRACTION, 1, 1,
- *       "width", G_TYPE_INT, 320, 
- *       "height", G_TYPE_INT, 240, 
+ *       "width", G_TYPE_INT, 320,
+ *       "height", G_TYPE_INT, 240,
  *       NULL);
  *  </programlisting>
  * </example>
@@ -60,7 +60,7 @@
  * Various methods exist to work with the media types such as subtracting
  * or intersecting.
  *
- * Last reviewed on 2005-11-23 (0.9.5)
+ * Last reviewed on 2007-02-13 (0.10.10)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -91,6 +91,14 @@
 #define IS_WRITABLE(caps) \
   (g_atomic_int_get (&(caps)->refcount) == 1)
 
+/* quick way to get a caps structure at an index without doing a type or array
+ * length check */
+#define gst_caps_get_structure_unchecked(caps, index) \
+     ((GstStructure *)g_ptr_array_index ((caps)->structs, (index)))
+
+
+/* lock to protect multiple invocations of static caps to caps conversion */
+G_LOCK_DEFINE_STATIC (static_caps_lock);
 
 static void gst_caps_transform_to_string (const GValue * src_value,
     GValue * dest_value);
@@ -129,10 +137,11 @@ gst_caps_get_type (void)
 GstCaps *
 gst_caps_new_empty (void)
 {
-  GstCaps *caps = g_new0 (GstCaps, 1);
+  GstCaps *caps = g_slice_new (GstCaps);
 
-  g_atomic_int_inc (&caps->refcount);
   caps->type = GST_TYPE_CAPS;
+  caps->refcount = 1;
+  caps->flags = 0;
   caps->structs = g_ptr_array_new ();
 
 #ifdef DEBUG_REFCOUNT
@@ -269,7 +278,7 @@ gst_caps_copy (const GstCaps * caps)
   newcaps->flags = caps->flags;
 
   for (i = 0; i < caps->structs->len; i++) {
-    structure = gst_caps_get_structure (caps, i);
+    structure = gst_caps_get_structure_unchecked (caps, i);
     gst_caps_append_structure (newcaps, gst_structure_copy (structure));
   }
 
@@ -286,7 +295,7 @@ _gst_caps_free (GstCaps * caps)
    * don't bother testing. */
 
   for (i = 0; i < caps->structs->len; i++) {
-    structure = (GstStructure *) gst_caps_get_structure (caps, i);
+    structure = (GstStructure *) gst_caps_get_structure_unchecked (caps, i);
     gst_structure_set_parent_refcount (structure, NULL);
     gst_structure_free (structure);
   }
@@ -294,7 +303,11 @@ _gst_caps_free (GstCaps * caps)
 #ifdef USE_POISONING
   memset (caps, 0xff, sizeof (GstCaps));
 #endif
-  g_free (caps);
+
+#ifdef DEBUG_REFCOUNT
+  GST_CAT_LOG (GST_CAT_CAPS, "freeing caps %p", caps);
+#endif
+  g_slice_free (GstCaps, caps);
 }
 
 /**
@@ -384,9 +397,8 @@ gst_caps_unref (GstCaps * caps)
   g_return_if_fail (GST_CAPS_REFCOUNT_VALUE (caps) > 0);
 
   /* if we ended up with the refcount at zero, free the caps */
-  if (g_atomic_int_dec_and_test (&caps->refcount)) {
+  if (G_UNLIKELY (g_atomic_int_dec_and_test (&caps->refcount)))
     _gst_caps_free (caps);
-  }
 }
 
 GType
@@ -414,43 +426,157 @@ gst_static_caps_get_type (void)
 GstCaps *
 gst_static_caps_get (GstStaticCaps * static_caps)
 {
-  GstCaps *caps = (GstCaps *) static_caps;
-  gboolean ret;
+  GstCaps *caps;
 
-  if (caps->type == 0) {
-    if (static_caps->string == NULL) {
-      g_warning ("static caps is NULL");
-      return NULL;
-    }
+  g_return_val_if_fail (static_caps != NULL, NULL);
 
-    caps->type = GST_TYPE_CAPS;
-    /* initialize the caps to a refcount of 1 so the caps can be writable... */
-    gst_atomic_int_set (&caps->refcount, 1);
-    caps->structs = g_ptr_array_new ();
-    ret = gst_caps_from_string_inplace (caps, static_caps->string);
+  caps = (GstCaps *) static_caps;
 
-    if (!ret) {
-      g_critical ("Could not convert static caps \"%s\"", static_caps->string);
-    }
+  /* refcount is 0 when we need to convert */
+  if (G_UNLIKELY (g_atomic_int_get (&caps->refcount) == 0)) {
+    const char *string;
+    GstCaps temp;
+
+    G_LOCK (static_caps_lock);
+    /* check if other thread already updated */
+    if (G_UNLIKELY (g_atomic_int_get (&caps->refcount) > 0))
+      goto done;
+
+    string = static_caps->string;
+
+    if (G_UNLIKELY (string == NULL))
+      goto no_string;
+
+    GST_CAT_LOG (GST_CAT_CAPS, "creating %p", static_caps);
+
+    /* we construct the caps on the stack, then copy over the struct into our
+     * real caps, refcount last. We do this because we must leave the refcount
+     * of the result caps to 0 so that other threads don't run away with the
+     * caps while we are constructing it. */
+    temp.type = GST_TYPE_CAPS;
+    temp.flags = 0;
+    temp.structs = g_ptr_array_new ();
+
+    /* initialize the caps to a refcount of 1 so the caps can be writable for
+     * the next statement */
+    temp.refcount = 1;
+
+    /* convert to string */
+    if (G_UNLIKELY (!gst_caps_from_string_inplace (&temp, string)))
+      g_critical ("Could not convert static caps \"%s\"", string);
+
+    /* now copy stuff over to the real caps. */
+    caps->type = temp.type;
+    caps->flags = temp.flags;
+    caps->structs = temp.structs;
+    /* and bump the refcount so other threads can now read */
+    g_atomic_int_set (&caps->refcount, 1);
+
+    GST_CAT_LOG (GST_CAT_CAPS, "created %p", static_caps);
+  done:
+    G_UNLOCK (static_caps_lock);
   }
   /* ref the caps, makes it not writable */
   gst_caps_ref (caps);
 
   return caps;
+
+  /* ERRORS */
+no_string:
+  {
+    G_UNLOCK (static_caps_lock);
+    g_warning ("static caps %p string is NULL", static_caps);
+    return NULL;
+  }
 }
 
 /* manipulation */
-
 static GstStructure *
 gst_caps_remove_and_get_structure (GstCaps * caps, guint idx)
 {
-  /* don't use index_fast, gst_caps_simplify relies on the order */
+  /* don't use index_fast, gst_caps_do_simplify relies on the order */
   GstStructure *s = g_ptr_array_remove_index (caps->structs, idx);
 
   gst_structure_set_parent_refcount (s, NULL);
   return s;
 }
 
+static gboolean
+gst_structure_is_equal_foreach (GQuark field_id, const GValue * val2,
+    gpointer data)
+{
+  GstStructure *struct1 = (GstStructure *) data;
+  const GValue *val1 = gst_structure_id_get_value (struct1, field_id);
+
+  if (val1 == NULL)
+    return FALSE;
+  if (gst_value_compare (val1, val2) == GST_VALUE_EQUAL) {
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
+static gboolean
+gst_caps_structure_is_subset_field (GQuark field_id, const GValue * value,
+    gpointer user_data)
+{
+  GstStructure *subtract_from = user_data;
+  GValue subtraction = { 0, };
+  const GValue *other;
+
+  if (!(other = gst_structure_id_get_value (subtract_from, field_id)))
+    /* field is missing in one set */
+    return FALSE;
+
+  /* equal values are subset */
+  if (gst_value_compare (other, value) == GST_VALUE_EQUAL)
+    return TRUE;
+
+  /*
+   * 1 - [1,2] = empty
+   * -> !subset
+   *
+   * [1,2] - 1 = 2
+   *  -> 1 - [1,2] = empty
+   *  -> subset
+   *
+   * [1,3] - [1,2] = 3
+   * -> [1,2] - [1,3] = empty
+   * -> subset
+   *
+   * {1,2} - {1,3} = 2
+   * -> {1,3} - {1,2} = 3
+   * -> !subset
+   *
+   *  First caps subtraction needs to return a non-empty set, second
+   *  subtractions needs to give en empty set.
+   */
+  if (gst_value_subtract (&subtraction, other, value)) {
+    g_value_unset (&subtraction);
+    /* !empty result, swapping must be empty */
+    if (!gst_value_subtract (&subtraction, value, other))
+      return TRUE;
+
+    g_value_unset (&subtraction);
+  }
+  return FALSE;
+}
+
+static gboolean
+gst_caps_structure_is_subset (const GstStructure * minuend,
+    const GstStructure * subtrahend)
+{
+  if ((minuend->name != subtrahend->name) ||
+      (gst_structure_n_fields (minuend) !=
+          gst_structure_n_fields (subtrahend))) {
+    return FALSE;
+  }
+
+  return gst_structure_foreach ((GstStructure *) subtrahend,
+      gst_caps_structure_is_subset_field, (gpointer) minuend);
+}
+
 /**
  * gst_caps_append:
  * @caps1: the #GstCaps that will be appended to
@@ -492,6 +618,63 @@ gst_caps_append (GstCaps * caps1, GstCaps * caps2)
   gst_caps_unref (caps2);       /* guaranteed to free it */
 }
 
+/**
+ * gst_caps_merge:
+ * @caps1: the #GstCaps that will take the new entries
+ * @caps2: the #GstCaps to merge in
+ *
+ * Appends the structures contained in @caps2 to @caps1 if they are not yet
+ * expressed by @caps1. The structures in @caps2 are not copied -- they are
+ * transferred to @caps1, and then @caps2 is freed.
+ * If either caps is ANY, the resulting caps will be ANY.
+ *
+ * Since: 0.10.10
+ */
+void
+gst_caps_merge (GstCaps * caps1, GstCaps * caps2)
+{
+  GstStructure *structure;
+  int i;
+
+  g_return_if_fail (GST_IS_CAPS (caps1));
+  g_return_if_fail (GST_IS_CAPS (caps2));
+  g_return_if_fail (IS_WRITABLE (caps1));
+  g_return_if_fail (IS_WRITABLE (caps2));
+
+#ifdef USE_POISONING
+  CAPS_POISON (caps2);
+#endif
+  if (gst_caps_is_any (caps1)) {
+    for (i = caps2->structs->len - 1; i >= 0; i--) {
+      structure = gst_caps_remove_and_get_structure (caps2, i);
+      gst_structure_free (structure);
+    }
+  } else if (gst_caps_is_any (caps2)) {
+    caps1->flags |= GST_CAPS_FLAGS_ANY;
+    for (i = caps1->structs->len - 1; i >= 0; i--) {
+      structure = gst_caps_remove_and_get_structure (caps1, i);
+      gst_structure_free (structure);
+    }
+  } else {
+    int len = caps2->structs->len;
+
+    for (i = 0; i < len; i++) {
+      structure = gst_caps_remove_and_get_structure (caps2, 0);
+      gst_caps_merge_structure (caps1, structure);
+    }
+    /* this is too naive
+       GstCaps *com = gst_caps_intersect (caps1, caps2);
+       GstCaps *add = gst_caps_subtract (caps2, com);
+
+       GST_DEBUG ("common : %d", gst_caps_get_size (com));
+       GST_DEBUG ("adding : %d", gst_caps_get_size (add));
+       gst_caps_append (caps1, add);
+       gst_caps_unref (com);
+     */
+  }
+  gst_caps_unref (caps2);       /* guaranteed to free it */
+}
+
 /**
  * gst_caps_append_structure:
  * @caps: the #GstCaps that will be appended to
@@ -506,7 +689,7 @@ gst_caps_append_structure (GstCaps * caps, GstStructure * structure)
   g_return_if_fail (GST_IS_CAPS (caps));
   g_return_if_fail (IS_WRITABLE (caps));
 
-  if (structure) {
+  if (G_LIKELY (structure)) {
     g_return_if_fail (structure->parent_refcount == NULL);
 #if 0
 #ifdef USE_POISONING
@@ -518,7 +701,7 @@ gst_caps_append_structure (GstCaps * caps, GstStructure * structure)
   }
 }
 
-/*
+/**
  * gst_caps_remove_structure:
  * @caps: the #GstCaps to remove from
  * @idx: Index of the structure to remove
@@ -540,22 +723,49 @@ gst_caps_remove_structure (GstCaps * caps, guint idx)
 }
 
 /**
- * gst_caps_split_one:
- * @caps: a #GstCaps
+ * gst_caps_merge_structure:
+ * @caps: the #GstCaps that will the the new structure
+ * @structure: the #GstStructure to merge
  *
- * This function is not implemented.
- *
- * Returns: NULL
+ * Appends @structure to @caps if its not already expressed by @caps.  The
+ * structure is not copied; @caps becomes the owner of @structure.
  */
-GstCaps *
-gst_caps_split_one (GstCaps * caps)
+void
+gst_caps_merge_structure (GstCaps * caps, GstStructure * structure)
 {
-  /* FIXME */
-  g_critical ("unimplemented");
+  g_return_if_fail (GST_IS_CAPS (caps));
+  g_return_if_fail (IS_WRITABLE (caps));
 
-  return NULL;
+  if (G_LIKELY (structure)) {
+    GstStructure *structure1;
+    int i;
+    gboolean unique = TRUE;
+
+    g_return_if_fail (structure->parent_refcount == NULL);
+#if 0
+#ifdef USE_POISONING
+    STRUCTURE_POISON (structure);
+#endif
+#endif
+    /* check each structure */
+    for (i = caps->structs->len - 1; i >= 0; i--) {
+      structure1 = gst_caps_get_structure_unchecked (caps, i);
+      /* if structure is a subset of structure1, then skip it */
+      if (gst_caps_structure_is_subset (structure1, structure)) {
+        unique = FALSE;
+        break;
+      }
+    }
+    if (unique) {
+      gst_structure_set_parent_refcount (structure, &caps->refcount);
+      g_ptr_array_add (caps->structs, structure);
+    } else {
+      gst_structure_free (structure);
+    }
+  }
 }
 
+
 /**
  * gst_caps_get_size:
  * @caps: a #GstCaps
@@ -591,10 +801,9 @@ GstStructure *
 gst_caps_get_structure (const GstCaps * caps, guint index)
 {
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
-  g_return_val_if_fail (index >= 0, NULL);
   g_return_val_if_fail (index < caps->structs->len, NULL);
 
-  return g_ptr_array_index (caps->structs, index);
+  return gst_caps_get_structure_unchecked (caps, index);
 }
 
 /**
@@ -619,7 +828,7 @@ gst_caps_copy_nth (const GstCaps * caps, guint nth)
   newcaps->flags = caps->flags;
 
   if (caps->structs->len > nth) {
-    structure = gst_caps_get_structure (caps, nth);
+    structure = gst_caps_get_structure_unchecked (caps, nth);
     gst_caps_append_structure (newcaps, gst_structure_copy (structure));
   }
 
@@ -658,7 +867,7 @@ gst_caps_truncate (GstCaps * caps)
  * manner as gst_structure_set(), and be NULL-terminated.
  */
 void
-gst_caps_set_simple (GstCaps * caps, char *field, ...)
+gst_caps_set_simple (GstCaps * caps, const char *field, ...)
 {
   GstStructure *structure;
   va_list var_args;
@@ -667,7 +876,7 @@ gst_caps_set_simple (GstCaps * caps, char *field, ...)
   g_return_if_fail (caps->structs->len == 1);
   g_return_if_fail (IS_WRITABLE (caps));
 
-  structure = gst_caps_get_structure (caps, 0);
+  structure = gst_caps_get_structure_unchecked (caps, 0);
 
   va_start (var_args, field);
   gst_structure_set_valist (structure, field, var_args);
@@ -685,15 +894,15 @@ gst_caps_set_simple (GstCaps * caps, char *field, ...)
  * manner as gst_structure_set(), and be NULL-terminated.
  */
 void
-gst_caps_set_simple_valist (GstCaps * caps, char *field, va_list varargs)
+gst_caps_set_simple_valist (GstCaps * caps, const char *field, va_list varargs)
 {
   GstStructure *structure;
 
   g_return_if_fail (GST_IS_CAPS (caps));
-  g_return_if_fail (caps->structs->len != 1);
+  g_return_if_fail (caps->structs->len == 1);
   g_return_if_fail (IS_WRITABLE (caps));
 
-  structure = gst_caps_get_structure (caps, 0);
+  structure = gst_caps_get_structure_unchecked (caps, 0);
 
   gst_structure_set_valist (structure, field, varargs);
 }
@@ -762,27 +971,11 @@ gst_caps_is_fixed (const GstCaps * caps)
   if (caps->structs->len != 1)
     return FALSE;
 
-  structure = gst_caps_get_structure (caps, 0);
+  structure = gst_caps_get_structure_unchecked (caps, 0);
 
   return gst_structure_foreach (structure, gst_caps_is_fixed_foreach, NULL);
 }
 
-static gboolean
-gst_structure_is_equal_foreach (GQuark field_id, const GValue * val2,
-    gpointer data)
-{
-  GstStructure *struct1 = (GstStructure *) data;
-  const GValue *val1 = gst_structure_id_get_value (struct1, field_id);
-
-  if (val1 == NULL)
-    return FALSE;
-  if (gst_value_compare (val1, val2) == GST_VALUE_EQUAL) {
-    return TRUE;
-  }
-
-  return FALSE;
-}
-
 /**
  * gst_caps_is_equal_fixed:
  * @caps1: the #GstCaps to test
@@ -801,8 +994,8 @@ gst_caps_is_equal_fixed (const GstCaps * caps1, const GstCaps * caps2)
   g_return_val_if_fail (gst_caps_is_fixed (caps1), FALSE);
   g_return_val_if_fail (gst_caps_is_fixed (caps2), FALSE);
 
-  struct1 = gst_caps_get_structure (caps1, 0);
-  struct2 = gst_caps_get_structure (caps2, 0);
+  struct1 = gst_caps_get_structure_unchecked (caps1, 0);
+  struct2 = gst_caps_get_structure_unchecked (caps2, 0);
 
   if (struct1->name != struct2->name) {
     return FALSE;
@@ -840,11 +1033,11 @@ gst_caps_is_always_compatible (const GstCaps * caps1, const GstCaps * caps2)
  * @subset: a #GstCaps
  * @superset: a potentially greater #GstCaps
  *
- * Checks if all caps represented by @subset are also represented by @superset
+ * Checks if all caps represented by @subset are also represented by @superset.
  * <note>This function does not work reliably if optional properties for caps
  * are included on one caps and omitted on the other.</note>
  *
- * Returns: TRUE if @subset is a subset of @superset
+ * Returns: %TRUE if @subset is a subset of @superset
  */
 gboolean
 gst_caps_is_subset (const GstCaps * subset, const GstCaps * superset)
@@ -875,13 +1068,16 @@ gst_caps_is_subset (const GstCaps * subset, const GstCaps * superset)
  * <note>This function does not work reliably if optional properties for caps
  * are included on one caps and omitted on the other.</note>
  *
- * This function deals correctly with passing NULL for any of the caps. 
+ * This function deals correctly with passing NULL for any of the caps.
  *
- * Returns: TRUE if both caps are equal. 
+ * Returns: TRUE if both caps are equal.
  */
 gboolean
 gst_caps_is_equal (const GstCaps * caps1, const GstCaps * caps2)
 {
+  /* FIXME 0.11: NULL pointers are no valid Caps but indicate an error
+   * So there should be an assertion that caps1 and caps2 != NULL */
+
   /* NULL <-> NULL is allowed here */
   if (caps1 == caps2)
     return TRUE;
@@ -1025,9 +1221,15 @@ gst_caps_intersect (const GstCaps * caps1, const GstCaps * caps2)
   g_return_val_if_fail (GST_IS_CAPS (caps1), NULL);
   g_return_val_if_fail (GST_IS_CAPS (caps2), NULL);
 
-  if (gst_caps_is_empty (caps1) || gst_caps_is_empty (caps2)) {
+  /* caps are exactly the same pointers, just copy one caps */
+  if (caps1 == caps2)
+    return gst_caps_copy (caps1);
+
+  /* empty caps on either side, return empty */
+  if (gst_caps_is_empty (caps1) || gst_caps_is_empty (caps2))
     return gst_caps_new_empty ();
-  }
+
+  /* one of the caps is any, just copy the other caps */
   if (gst_caps_is_any (caps1))
     return gst_caps_copy (caps2);
   if (gst_caps_is_any (caps2))
@@ -1061,8 +1263,8 @@ gst_caps_intersect (const GstCaps * caps1, const GstCaps * caps2)
     /* now run the diagonal line, end condition is the left or bottom
      * border */
     while (k < caps2->structs->len) {
-      struct1 = gst_caps_get_structure (caps1, j);
-      struct2 = gst_caps_get_structure (caps2, k);
+      struct1 = gst_caps_get_structure_unchecked (caps1, j);
+      struct2 = gst_caps_get_structure_unchecked (caps2, k);
 
       istruct = gst_caps_structure_intersect (struct1, struct2);
 
@@ -1175,14 +1377,14 @@ gst_caps_subtract (const GstCaps * minuend, const GstCaps * subtrahend)
 
   src = gst_caps_copy (minuend);
   for (i = 0; i < subtrahend->structs->len; i++) {
-    sub = gst_caps_get_structure (subtrahend, i);
+    sub = gst_caps_get_structure_unchecked (subtrahend, i);
     if (dest) {
       gst_caps_unref (src);
       src = dest;
     }
     dest = gst_caps_new_empty ();
     for (j = 0; j < src->structs->len; j++) {
-      min = gst_caps_get_structure (src, j);
+      min = gst_caps_get_structure_unchecked (src, j);
       if (gst_structure_get_name_id (min) == gst_structure_get_name_id (sub)) {
         GSList *list;
 
@@ -1227,6 +1429,16 @@ gst_caps_union (const GstCaps * caps1, const GstCaps * caps2)
   GstCaps *dest1;
   GstCaps *dest2;
 
+  /* NULL pointers are no correct GstCaps */
+  g_return_val_if_fail (caps1 != NULL, NULL);
+  g_return_val_if_fail (caps2 != NULL, NULL);
+
+  if (gst_caps_is_empty (caps1))
+    return gst_caps_copy (caps2);
+
+  if (gst_caps_is_empty (caps2))
+    return gst_caps_copy (caps1);
+
   if (gst_caps_is_any (caps1) || gst_caps_is_any (caps2))
     return gst_caps_new_any ();
 
@@ -1293,7 +1505,7 @@ gst_caps_normalize (const GstCaps * caps)
   nf.caps = newcaps;
 
   for (i = 0; i < newcaps->structs->len; i++) {
-    nf.structure = gst_caps_get_structure (newcaps, i);
+    nf.structure = gst_caps_get_structure_unchecked (newcaps, i);
 
     while (!gst_structure_foreach (nf.structure,
             gst_caps_normalize_foreach, &nf));
@@ -1309,7 +1521,7 @@ gst_caps_compare_structures (gconstpointer one, gconstpointer two)
   const GstStructure *struct1 = *((const GstStructure **) one);
   const GstStructure *struct2 = *((const GstStructure **) two);
 
-  /* FIXME: this orders aphabetically, but ordering the quarks might be faster
+  /* FIXME: this orders alphabetically, but ordering the quarks might be faster
      So what's the best way? */
   ret = strcmp (gst_structure_get_name (struct1),
       gst_structure_get_name (struct2));
@@ -1319,30 +1531,6 @@ gst_caps_compare_structures (gconstpointer one, gconstpointer two)
   return gst_structure_n_fields (struct2) - gst_structure_n_fields (struct1);
 }
 
-/**
- * gst_caps_simplify:
- * @caps: a #GstCaps to simplify
- *
- * Creates a new #GstCaps that represents the same set of formats as
- * @caps, but simpler.  Component structures that are identical are
- * merged.  Component structures that have ranges or lists that can
- * be merged are also merged.
- *
- * Returns: the new #GstCaps
- */
-GstCaps *
-gst_caps_simplify (const GstCaps * caps)
-{
-  GstCaps *ret;
-
-  g_return_val_if_fail (caps != NULL, NULL);
-
-  ret = gst_caps_copy (caps);
-  gst_caps_do_simplify (ret);
-
-  return ret;
-}
-
 typedef struct
 {
   GQuark name;
@@ -1478,14 +1666,15 @@ gst_caps_do_simplify (GstCaps * caps)
 
   start = caps->structs->len - 1;
   for (i = caps->structs->len - 1; i >= 0; i--) {
-    simplify = gst_caps_get_structure (caps, i);
+    simplify = gst_caps_get_structure_unchecked (caps, i);
     if (gst_structure_get_name_id (simplify) !=
-        gst_structure_get_name_id (gst_caps_get_structure (caps, start)))
+        gst_structure_get_name_id (gst_caps_get_structure_unchecked (caps,
+                start)))
       start = i;
     for (j = start; j >= 0; j--) {
       if (j == i)
         continue;
-      compare = gst_caps_get_structure (caps, j);
+      compare = gst_caps_get_structure_unchecked (caps, j);
       if (gst_structure_get_name_id (simplify) !=
           gst_structure_get_name_id (compare)) {
         break;
@@ -1560,26 +1749,28 @@ gst_caps_load_thyself (xmlNodePtr parent)
  * Replaces *caps with @newcaps.  Unrefs the #GstCaps in the location
  * pointed to by @caps, if applicable, then modifies @caps to point to
  * @newcaps. An additional ref on @newcaps is taken.
+ *
+ * This function does not take any locks so you might want to lock
+ * the object owning @caps pointer.
  */
 void
 gst_caps_replace (GstCaps ** caps, GstCaps * newcaps)
 {
   GstCaps *oldcaps;
 
-#if 0                           /* disable this, since too many plugins rely on undefined behavior */
-#ifdef USE_POISONING
-  //if (newcaps) CAPS_POISON (newcaps);
-#endif
-#endif
+  g_return_if_fail (caps != NULL);
+
   oldcaps = *caps;
 
-  if (newcaps)
-    gst_caps_ref (newcaps);
+  if (newcaps != oldcaps) {
+    if (newcaps)
+      gst_caps_ref (newcaps);
 
-  *caps = newcaps;
+    *caps = newcaps;
 
-  if (oldcaps)
-    gst_caps_unref (oldcaps);
+    if (oldcaps)
+      gst_caps_unref (oldcaps);
+  }
 }
 
 /**
@@ -1600,7 +1791,7 @@ gst_caps_replace (GstCaps ** caps, GstCaps * newcaps)
 gchar *
 gst_caps_to_string (const GstCaps * caps)
 {
-  guint i;
+  guint i, slen;
   GString *s;
 
   /* NOTE:  This function is potentially called by the debug system,
@@ -1619,20 +1810,30 @@ gst_caps_to_string (const GstCaps * caps)
     return g_strdup ("EMPTY");
   }
 
-  s = g_string_new ("");
+  /* estimate a rough string length to avoid unnecessary reallocs in GString */
+  slen = 0;
   for (i = 0; i < caps->structs->len; i++) {
-    GstStructure *structure;
-    char *sstr;
-
-    if (i > 0)
-      g_string_append (s, "; ");
-
-    structure = gst_caps_get_structure (caps, i);
-    sstr = gst_structure_to_string (structure);
-    g_string_append (s, sstr);
-    g_free (sstr);
+    slen +=
+        STRUCTURE_ESTIMATED_STRING_LEN (gst_caps_get_structure_unchecked (caps,
+            i));
   }
 
+  s = g_string_sized_new (slen);
+  for (i = 0; i < caps->structs->len; i++) {
+    GstStructure *structure;
+
+    if (i > 0) {
+      /* ';' is now added by gst_structure_to_string */
+      g_string_append_c (s, ' ');
+    }
+
+    structure = gst_caps_get_structure_unchecked (caps, i);
+    priv_gst_structure_append_to_gstring (structure, s);
+  }
+  if (s->len && s->str[s->len - 1] == ';') {
+    /* remove latest ';' */
+    s->str[--s->len] = '\0';
+  }
   return g_string_free (s, FALSE);
 }
 
@@ -1657,22 +1858,20 @@ gst_caps_from_string_inplace (GstCaps * caps, const gchar * string)
   }
   gst_caps_append_structure (caps, structure);
 
-  while (*s == ';') {
-    s++;
+  do {
+
     while (g_ascii_isspace (*s))
       s++;
+    if (*s == '\0') {
+      break;
+    }
     structure = gst_structure_from_string (s, &s);
     if (structure == NULL) {
       return FALSE;
     }
     gst_caps_append_structure (caps, structure);
-    while (g_ascii_isspace (*s))
-      s++;
-  }
 
-  if (*s != 0) {
-    return FALSE;
-  }
+  } while (TRUE);
 
   return TRUE;
 }
