@@ -30,26 +30,33 @@ G_BEGIN_DECLS
 #define GST_TYPE_IMPLEMENTS_INTERFACE \
   (gst_implements_interface_get_type ())
 #define GST_IMPLEMENTS_INTERFACE(obj) \
-  (GST_INTERFACE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_IMPLEMENTS_INTERFACE, \
+  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_IMPLEMENTS_INTERFACE, \
 				      GstImplementsInterface))
 #define GST_IMPLEMENTS_INTERFACE_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_IMPLEMENTS_INTERFACE, \
 			    GstImplementsInterfaceClass))
 #define GST_IS_IMPLEMENTS_INTERFACE(obj) \
-  (GST_INTERFACE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_IMPLEMENTS_INTERFACE))
+  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_IMPLEMENTS_INTERFACE))
 #define GST_IS_IMPLEMENTS_INTERFACE_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_IMPLEMENTS_INTERFACE))
 #define GST_IMPLEMENTS_INTERFACE_GET_CLASS(inst) \
   (G_TYPE_INSTANCE_GET_INTERFACE ((inst), GST_TYPE_IMPLEMENTS_INTERFACE, \
 				  GstImplementsInterfaceClass))
 
+/**
+ * GstImplementsInterface:
+ *
+ * Opaque #GstImplementsInterface structure.
+ */
 typedef struct _GstImplementsInterface GstImplementsInterface;
 
+typedef struct _GstImplementsInterfaceClass GstImplementsInterfaceClass;
+
 /* This small extra virtual function is here to provide an
  * interface functionality on a per-instance basis rather
  * than a per-class basis, which is the case for glib.
  */
-typedef struct _GstImplementsInterfaceClass {
+struct _GstImplementsInterfaceClass {
   GTypeInterface parent;
 
   /* virtual functions */
@@ -57,7 +64,7 @@ typedef struct _GstImplementsInterfaceClass {
 			  GType                   iface_type);
 
   gpointer _gst_reserved[GST_PADDING];
-} GstImplementsInterfaceClass;
+};
 
 #define GST_IMPLEMENTS_INTERFACE_CHECK_INSTANCE_CAST(obj, type, cast_t) \
   ((cast_t *) gst_implements_interface_cast ((obj), (type)))
