@@ -2,6 +2,7 @@
  * Copyright (C) 2006 Josep Torra <josep@fluendo.com>
  *               2006 Mathieu Garcia <matthieu@fluendo.com>
  *               2006,2007 Stefan Kost <ensonic@users.sf.net>
+ *               2008 Sebastian Dröge <slomo@circular-chaos.org>
  *
  * gstregistrybinary.c: GstRegistryBinary object, support routines
  *
@@ -22,14 +23,12 @@
  */
 
 /* FIXME:
- * - Add random key to libgstreamer during build and only accept registry,
- *   if key matches (or is the version check enough)
  * - keep registry binary blob and reference strings
  *   - don't free/unmmap contents when leaving gst_registry_binary_read_cache()
  *     - free at gst_deinit() / _priv_gst_registry_cleanup() ?
  *   - GstPlugin:
  *     - GST_PLUGIN_FLAG_CONST
- *   -GstPluginFeature, GstIndexFactory, GstElementFactory
+ *   - GstPluginFeature, GstIndexFactory, GstElementFactory
  *     - needs Flags (GST_PLUGIN_FEATURE_FLAG_CONST)
  *     - can we turn loaded into flag?
  * - why do we collect a list of binary chunks and not write immediately
@@ -37,7 +36,6 @@
  *     in parent chunk
  * - need more robustness
  *   - don't parse beyond mem-block size
- *   - include md5-sum ?
  */
 
 #ifdef HAVE_CONFIG_H
@@ -76,60 +74,226 @@
 
 /* macros */
 
-#define unpack_element(_inptr, _outptr, _element)  \
-  _outptr = (_element *)_inptr; \
-  _inptr += sizeof (_element)
+#define unpack_element(_inptr, _outptr, _element)  G_STMT_START{ \
+  _outptr = (_element *) _inptr; \
+  _inptr += sizeof (_element); \
+}G_STMT_END
 
-#define unpack_const_string(_inptr, _outptr) \
+#define unpack_const_string(_inptr, _outptr) G_STMT_START{\
   _outptr = g_intern_string ((const gchar *)_inptr); \
-  _inptr += strlen(_outptr) + 1
+  _inptr += strlen(_outptr) + 1; \
+}G_STMT_END
 
-#define unpack_string(_inptr, _outptr)  \
+#define unpack_string(_inptr, _outptr)  G_STMT_START{\
   _outptr = g_strdup ((gchar *)_inptr); \
-  _inptr += strlen(_outptr) + 1
-
-#if !GST_HAVE_UNALIGNED_ACCESS
-#  define alignment32(_address)  (gsize)_address%4
-#  define align32(_ptr)          _ptr += (( alignment32(_ptr) == 0) ? 0 : 4-alignment32(_ptr))
-#else
-#  define alignment32(_address)  0
-#  define align32(_ptr)          do {} while(0)
-#endif
+  _inptr += strlen(_outptr) + 1; \
+}G_STMT_END
 
+#define ALIGNMENT            (sizeof (void *))
+#define alignment(_address)  (gsize)_address%ALIGNMENT
+#define align(_ptr)          _ptr += (( alignment(_ptr) == 0) ? 0 : ALIGNMENT-alignment(_ptr))
 
 /* Registry saving */
 
+#ifdef G_OS_WIN32
+/* On win32, we can't use g_mkstmp(), because of cross-DLL file I/O problems.
+ * So, we just create the entire binary registry in memory, then write it out
+ * with g_file_set_contents(), which creates a temporary file internally
+ */
+
+typedef struct BinaryRegistryCache
+{
+  const char *location;
+  guint8 *mem;
+  gssize len;
+} BinaryRegistryCache;
+
+static BinaryRegistryCache *
+gst_registry_binary_cache_init (GstRegistry * registry, const char *location)
+{
+  BinaryRegistryCache *cache = g_new0 (BinaryRegistryCache, 1);
+  cache->location = location;
+  return cache;
+}
+
+static int
+gst_registry_binary_cache_write (GstRegistry * registry,
+    BinaryRegistryCache * cache, unsigned long offset,
+    const void *data, int length)
+{
+  cache->len = MAX (offset + length, cache->len);
+  cache->mem = g_realloc (cache->mem, cache->len);
+
+  memcpy (cache->mem + offset, data, length);
+
+  return length;
+}
+
+static gboolean
+gst_registry_binary_cache_finish (GstRegistry * registry,
+    BinaryRegistryCache * cache, gboolean success)
+{
+  gboolean ret = TRUE;
+  GError *error = NULL;
+  if (!g_file_set_contents (cache->location, (const gchar *) cache->mem,
+          cache->len, &error)) {
+    /* Probably the directory didn't exist; create it */
+    gchar *dir;
+    dir = g_path_get_dirname (cache->location);
+    g_mkdir_with_parents (dir, 0777);
+    g_free (dir);
+
+    g_error_free (error);
+    error = NULL;
+
+    if (!g_file_set_contents (cache->location, (const gchar *) cache->mem,
+            cache->len, &error)) {
+      GST_ERROR ("Failed to write to cache file: %s", error->message);
+      g_error_free (error);
+      ret = FALSE;
+    }
+  }
+
+  g_free (cache->mem);
+  g_free (cache);
+  return ret;
+}
+
+#else
+typedef struct BinaryRegistryCache
+{
+  const char *location;
+  char *tmp_location;
+  unsigned long currentoffset;
+} BinaryRegistryCache;
+
+static BinaryRegistryCache *
+gst_registry_binary_cache_init (GstRegistry * registry, const char *location)
+{
+  BinaryRegistryCache *cache = g_new0 (BinaryRegistryCache, 1);
+
+  cache->location = location;
+  cache->tmp_location = g_strconcat (location, ".tmpXXXXXX", NULL);
+  registry->cache_file = g_mkstemp (cache->tmp_location);
+  if (registry->cache_file == -1) {
+    gchar *dir;
+
+    /* oops, I bet the directory doesn't exist */
+    dir = g_path_get_dirname (location);
+    g_mkdir_with_parents (dir, 0777);
+    g_free (dir);
+
+    /* the previous g_mkstemp call overwrote the XXXXXX placeholder ... */
+    g_free (cache->tmp_location);
+    cache->tmp_location = g_strconcat (location, ".tmpXXXXXX", NULL);
+    registry->cache_file = g_mkstemp (cache->tmp_location);
+
+    if (registry->cache_file == -1) {
+      GST_DEBUG ("g_mkstemp() failed: %s", g_strerror (errno));
+      g_free (cache->tmp_location);
+      g_free (cache);
+      return NULL;
+    }
+  }
+
+  return cache;
+}
+
+static int
+gst_registry_binary_cache_write (GstRegistry * registry,
+    BinaryRegistryCache * cache, unsigned long offset,
+    const void *data, int length)
+{
+  long written;
+  if (offset != cache->currentoffset) {
+    if (lseek (registry->cache_file, offset, SEEK_SET) != 0) {
+      GST_ERROR ("Seeking to new offset failed");
+      return FALSE;
+    }
+    cache->currentoffset = offset;
+  }
+
+  written = write (registry->cache_file, data, length);
+  if (written != length) {
+    GST_ERROR ("Failed to write to cache file");
+  }
+  cache->currentoffset += written;
+
+  return written;
+}
+
+static gboolean
+gst_registry_binary_cache_finish (GstRegistry * registry,
+    BinaryRegistryCache * cache, gboolean success)
+{
+  if (close (registry->cache_file) < 0)
+    goto close_failed;
+
+  if (success) {
+    /* Only do the rename if we wrote the entire file successfully */
+    if (g_rename (cache->tmp_location, cache->location) < 0)
+      goto rename_failed;
+  }
+
+  g_free (cache->tmp_location);
+  g_free (cache);
+  GST_INFO ("Wrote binary registry cache");
+  return TRUE;
+
+fail_after_close:
+  {
+    g_unlink (cache->tmp_location);
+    g_free (cache->tmp_location);
+    g_free (cache);
+    return FALSE;
+  }
+close_failed:
+  {
+    GST_ERROR ("close() failed: %s", g_strerror (errno));
+    goto fail_after_close;
+  }
+rename_failed:
+  {
+    GST_ERROR ("g_rename() failed: %s", g_strerror (errno));
+    goto fail_after_close;
+  }
+}
+#endif
+
 /*
- * gst_registry_binary_write:
+ * gst_registry_binary_write_chunk:
  *
  * Write from a memory location to the registry cache file
  *
  * Returns: %TRUE for success
  */
 inline static gboolean
-gst_registry_binary_write (GstRegistry * registry, const void *mem,
+gst_registry_binary_write_chunk (GstRegistry * registry,
+    BinaryRegistryCache * cache, const void *mem,
     const gssize size, unsigned long *file_position, gboolean align)
 {
-#if !GST_HAVE_UNALIGNED_ACCESS
-  gchar padder[] = { 0, 0, 0, 0 };
+  gchar padder[ALIGNMENT] = { 0, };
   int padsize = 0;
 
   /* Padding to insert the struct that requiere word alignment */
-  if ((align) && (alignment32 (*file_position) != 0)) {
-    padsize = 4 - alignment32 (*file_position);
-    if (write (registry->cache_file, padder, padsize) != padsize) {
+  if ((align) && (alignment (*file_position) != 0)) {
+    padsize = ALIGNMENT - alignment (*file_position);
+    if (gst_registry_binary_cache_write (registry, cache, *file_position,
+            padder, padsize) != padsize) {
       GST_ERROR ("Failed to write binary registry padder");
       return FALSE;
     }
-    *file_position = *file_position + padsize;
+    *file_position += padsize;
   }
-#endif
 
-  if (write (registry->cache_file, mem, size) != size) {
+  if (gst_registry_binary_cache_write (registry, cache, *file_position,
+          mem, size) != size) {
     GST_ERROR ("Failed to write binary registry element");
     return FALSE;
   }
-  *file_position = *file_position + size;
+
+  *file_position += size;
+
   return TRUE;
 }
 
@@ -143,12 +307,16 @@ gst_registry_binary_write (GstRegistry * registry, const void *mem,
 inline static gboolean
 gst_registry_binary_initialize_magic (GstBinaryRegistryMagic * m)
 {
+  memset (m, 0, sizeof (GstBinaryRegistryMagic));
+
   if (!strncpy (m->magic, GST_MAGIC_BINARY_REGISTRY_STR,
           GST_MAGIC_BINARY_REGISTRY_LEN)
-      || !strncpy (m->version, GST_MAJORMINOR, GST_MAGIC_BINARY_VERSION_LEN)) {
+      || !strncpy (m->version, GST_MAGIC_BINARY_VERSION_STR,
+          GST_MAGIC_BINARY_VERSION_LEN)) {
     GST_ERROR ("Failed to write magic to the registry magic structure");
     return FALSE;
   }
+
   return TRUE;
 }
 
@@ -165,6 +333,11 @@ gst_registry_binary_save_const_string (GList ** list, const gchar * str)
 {
   GstBinaryChunk *chunk;
 
+  if (G_UNLIKELY (str == NULL)) {
+    GST_ERROR ("unexpected NULL string in plugin or plugin feature data");
+    str = "";
+  }
+
   chunk = g_malloc (sizeof (GstBinaryChunk));
   chunk->data = (gpointer) str;
   chunk->size = strlen ((gchar *) chunk->data) + 1;
@@ -285,7 +458,7 @@ gst_registry_binary_save_feature (GList ** list, GstPluginFeature * feature)
     GST_DEBUG ("Saved %d Interfaces", ef->ninterfaces);
     /* save uritypes */
     if (GST_URI_TYPE_IS_VALID (factory->uri_type)) {
-      if (factory->uri_protocols) {
+      if (factory->uri_protocols && *factory->uri_protocols) {
         GstBinaryChunk *subchk;
         gchar **protocol;
 
@@ -350,9 +523,7 @@ gst_registry_binary_save_feature (GList ** list, GstPluginFeature * feature)
     str = gst_caps_to_string (copy);
     gst_caps_unref (copy);
     gst_registry_binary_save_string (list, str);
-  }
-#ifndef GST_DISABLE_INDEX
-  else if (GST_IS_INDEX_FACTORY (feature)) {
+  } else if (GST_IS_INDEX_FACTORY (feature)) {
     GstIndexFactory *factory = GST_INDEX_FACTORY (feature);
 
     pf = g_malloc (sizeof (GstBinaryPluginFeature));
@@ -361,9 +532,7 @@ gst_registry_binary_save_feature (GList ** list, GstPluginFeature * feature)
 
     /* pack element factory strings */
     gst_registry_binary_save_const_string (list, factory->longdesc);
-  }
-#endif
-  else {
+  } else {
     GST_WARNING ("unhandled feature type '%s'", type_name);
   }
 
@@ -385,6 +554,38 @@ fail:
   return FALSE;
 }
 
+static gboolean
+gst_registry_binary_save_plugin_dep (GList ** list, GstPluginDep * dep)
+{
+  GstBinaryDep *ed;
+  GstBinaryChunk *chk;
+  gchar **s;
+
+  ed = g_new0 (GstBinaryDep, 1);
+  chk = gst_registry_binary_make_data (ed, sizeof (GstBinaryDep));
+
+  ed->flags = dep->flags;
+  ed->n_env_vars = 0;
+  ed->n_paths = 0;
+  ed->n_names = 0;
+
+  ed->env_hash = dep->env_hash;
+  ed->stat_hash = dep->stat_hash;
+
+  for (s = dep->env_vars; s != NULL && *s != NULL; ++s, ++ed->n_env_vars)
+    gst_registry_binary_save_string (list, *s);
+
+  for (s = dep->paths; s != NULL && *s != NULL; ++s, ++ed->n_paths)
+    gst_registry_binary_save_string (list, *s);
+
+  for (s = dep->names; s != NULL && *s != NULL; ++s, ++ed->n_names)
+    gst_registry_binary_save_string (list, *s);
+
+  *list = g_list_prepend (*list, chk);
+
+  GST_LOG ("Saved external plugin dependency");
+  return TRUE;
+}
 
 /*
  * gst_registry_binary_save_plugin:
@@ -406,8 +607,18 @@ gst_registry_binary_save_plugin (GList ** list, GstRegistry * registry,
 
   pe->file_size = plugin->file_size;
   pe->file_mtime = plugin->file_mtime;
+  pe->n_deps = 0;
   pe->nfeatures = 0;
 
+  /* pack external deps */
+  for (walk = plugin->priv->deps; walk != NULL; walk = walk->next) {
+    if (!gst_registry_binary_save_plugin_dep (list, walk->data)) {
+      GST_ERROR ("Could not save external plugin dependency, aborting.");
+      goto fail;
+    }
+    ++pe->n_deps;
+  }
+
   /* pack plugin features */
   plugin_features =
       gst_registry_get_feature_list_by_plugin (registry, plugin->desc.name);
@@ -419,7 +630,7 @@ gst_registry_binary_save_plugin (GList ** list, GstRegistry * registry,
       goto fail;
     }
   }
-  GST_DEBUG ("Save plugin '%s' with %d features", plugin->desc.name,
+  GST_DEBUG ("Save plugin '%s' with %d feature(s)", plugin->desc.name,
       pe->nfeatures);
 
   gst_plugin_feature_list_free (plugin_features);
@@ -448,7 +659,6 @@ fail:
   return FALSE;
 }
 
-
 /**
  * gst_registry_binary_write_cache:
  * @registry: a #GstRegistry
@@ -462,47 +672,18 @@ gboolean
 gst_registry_binary_write_cache (GstRegistry * registry, const char *location)
 {
   GList *walk;
-  gchar *tmp_location;
-  GstBinaryRegistryMagic *magic;
-  GstBinaryChunk *magic_chunk;
+  GstBinaryRegistryMagic magic;
   GList *to_write = NULL;
   unsigned long file_position = 0;
+  BinaryRegistryCache *cache;
 
   GST_INFO ("Building binary registry cache image");
 
   g_return_val_if_fail (GST_IS_REGISTRY (registry), FALSE);
-  tmp_location = g_strconcat (location, ".tmpXXXXXX", NULL);
-  registry->cache_file = g_mkstemp (tmp_location);
-  if (registry->cache_file == -1) {
-    gchar *dir;
 
-    /* oops, I bet the directory doesn't exist */
-    dir = g_path_get_dirname (location);
-    g_mkdir_with_parents (dir, 0777);
-    g_free (dir);
-
-    /* the previous g_mkstemp call overwrote the XXXXXX placeholder ... */
-    g_free (tmp_location);
-    tmp_location = g_strconcat (location, ".tmpXXXXXX", NULL);
-    registry->cache_file = g_mkstemp (tmp_location);
-
-    if (registry->cache_file == -1) {
-      GST_DEBUG ("g_mkstemp() failed: %s", g_strerror (errno));
-      g_free (tmp_location);
-      return FALSE;
-    }
-  }
-
-  magic = g_malloc (sizeof (GstBinaryRegistryMagic));
-  if (!gst_registry_binary_initialize_magic (magic))
+  if (!gst_registry_binary_initialize_magic (&magic))
     goto fail;
 
-  magic_chunk = g_malloc (sizeof (GstBinaryChunk));
-  magic_chunk->data = magic;
-  magic_chunk->size = sizeof (GstBinaryRegistryMagic);
-  magic_chunk->flags = GST_BINARY_REGISTRY_FLAG_NONE;
-  magic_chunk->align = TRUE;
-
   /* iterate trough the list of plugins and fit them into binary structures */
   for (walk = registry->plugins; walk; walk = g_list_next (walk)) {
     GstPlugin *plugin = GST_PLUGIN (walk->data);
@@ -514,7 +695,6 @@ gst_registry_binary_write_cache (GstRegistry * registry, const char *location)
       int ret;
       struct stat statbuf;
 
-      ret = g_stat (plugin->filename, &statbuf);
       if ((ret = g_stat (plugin->filename, &statbuf)) < 0 ||
           plugin->file_mtime != statbuf.st_mtime ||
           plugin->file_size != statbuf.st_size)
@@ -526,67 +706,66 @@ gst_registry_binary_write_cache (GstRegistry * registry, const char *location)
           plugin->filename);
     }
   }
-  to_write = g_list_prepend (to_write, magic_chunk);
 
   GST_INFO ("Writing binary registry cache");
 
+  cache = gst_registry_binary_cache_init (registry, location);
+  if (!cache)
+    goto fail_free_list;
+
+  /* write magic */
+  if (gst_registry_binary_cache_write (registry, cache, file_position,
+          &magic, sizeof (GstBinaryRegistryMagic)) !=
+      sizeof (GstBinaryRegistryMagic)) {
+    GST_ERROR ("Failed to write binary registry magic");
+    goto fail_free_list;
+  }
+  file_position += sizeof (GstBinaryRegistryMagic);
+
   /* write out data chunks */
   for (walk = to_write; walk; walk = g_list_next (walk)) {
     GstBinaryChunk *cur = walk->data;
 
-    if (!gst_registry_binary_write (registry, cur->data, cur->size,
+    if (!gst_registry_binary_write_chunk (registry, cache, cur->data, cur->size,
             &file_position, cur->align)) {
       if (!(cur->flags & GST_BINARY_REGISTRY_FLAG_CONST))
         g_free (cur->data);
       g_free (cur);
-      g_list_free (to_write);
-      goto fail;
+      walk->data = NULL;
+      goto fail_free_list;
     }
     if (!(cur->flags & GST_BINARY_REGISTRY_FLAG_CONST))
       g_free (cur->data);
     g_free (cur);
+    walk->data = NULL;
   }
   g_list_free (to_write);
 
-  if (close (registry->cache_file) < 0)
-    goto close_failed;
+  if (!gst_registry_binary_cache_finish (registry, cache, TRUE))
+    return FALSE;
 
-  if (g_file_test (tmp_location, G_FILE_TEST_EXISTS)) {
-#ifdef WIN32
-    g_remove (location);
-#endif
-    if (g_rename (tmp_location, location) < 0)
-      goto rename_failed;
-  } else {
-    /* FIXME: shouldn't we return FALSE here? */
-  }
-
-  g_free (tmp_location);
-  GST_INFO ("Wrote binary registry cache");
   return TRUE;
 
   /* Errors */
-fail:
+fail_free_list:
   {
-    (void) close (registry->cache_file);
+    for (walk = to_write; walk; walk = g_list_next (walk)) {
+      GstBinaryChunk *cur = walk->data;
+
+      if (!(cur->flags & GST_BINARY_REGISTRY_FLAG_CONST))
+        g_free (cur->data);
+      g_free (cur);
+    }
+    g_list_free (to_write);
+
+    if (cache)
+      (void) gst_registry_binary_cache_finish (registry, cache, FALSE);
     /* fall through */
   }
-fail_after_close:
+fail:
   {
-    g_remove (tmp_location);
-    g_free (tmp_location);
     return FALSE;
   }
-close_failed:
-  {
-    GST_ERROR ("close() failed: %s", g_strerror (errno));
-    goto fail_after_close;
-  }
-rename_failed:
-  {
-    GST_ERROR ("g_rename() failed: %s", g_strerror (errno));
-    goto fail_after_close;
-  }
 }
 
 
@@ -596,20 +775,22 @@ rename_failed:
  * gst_registry_binary_check_magic:
  *
  * Check GstBinaryRegistryMagic validity.
- * Return FALSE if something is wrong
+ * Return < 0 if something is wrong, -2 means
+ * that just the version of the registry is out of
+ * date, -1 is a general failure.
  */
-static gboolean
-gst_registry_binary_check_magic (gchar ** in)
+static gint
+gst_registry_binary_check_magic (gchar ** in, gsize size)
 {
   GstBinaryRegistryMagic *m;
 
-  align32 (*in);
+  align (*in);
   GST_DEBUG ("Reading/casting for GstBinaryRegistryMagic at address %p", *in);
   unpack_element (*in, m, GstBinaryRegistryMagic);
 
   if (m == NULL || m->magic == NULL || m->version == NULL) {
     GST_WARNING ("Binary registry magic structure is broken");
-    return FALSE;
+    return -1;
   }
   if (strncmp (m->magic, GST_MAGIC_BINARY_REGISTRY_STR,
           GST_MAGIC_BINARY_REGISTRY_LEN) != 0) {
@@ -620,14 +801,16 @@ gst_registry_binary_check_magic (gchar ** in)
         GST_MAGIC_BINARY_REGISTRY_STR[2] & 0xff,
         GST_MAGIC_BINARY_REGISTRY_STR[3] & 0xff, m->magic[0] & 0xff,
         m->magic[1] & 0xff, m->magic[2] & 0xff, m->magic[3] & 0xff);
-    return FALSE;
+    return -1;
   }
-  if (strncmp (m->version, GST_MAJORMINOR, GST_MAGIC_BINARY_VERSION_LEN)) {
+  if (strncmp (m->version, GST_MAGIC_BINARY_VERSION_STR,
+          GST_MAGIC_BINARY_VERSION_LEN)) {
     GST_WARNING ("Binary registry magic version is different : %s != %s",
-        GST_MAJORMINOR, m->version);
-    return FALSE;
+        GST_MAGIC_BINARY_VERSION_STR, m->version);
+    return -2;
   }
-  return TRUE;
+
+  return 0;
 }
 
 
@@ -644,7 +827,7 @@ gst_registry_binary_load_pad_template (GstElementFactory * factory, gchar ** in)
   GstBinaryPadTemplate *pt;
   GstStaticPadTemplate *template;
 
-  align32 (*in);
+  align (*in);
   GST_DEBUG ("Reading/casting for GstBinaryPadTemplate at address %p", *in);
   unpack_element (*in, pt, GstBinaryPadTemplate);
 
@@ -683,8 +866,10 @@ gst_registry_binary_load_feature (GstRegistry * registry, gchar ** in,
   /* unpack plugin feature strings */
   unpack_string (*in, type_name);
 
-  if (!type_name || !*(type_name))
+  if (!type_name || !*(type_name)) {
+    GST_ERROR ("No feature type name");
     return FALSE;
+  }
 
   GST_DEBUG ("Plugin '%s' feature typename : '%s'", plugin_name, type_name);
 
@@ -699,7 +884,7 @@ gst_registry_binary_load_feature (GstRegistry * registry, gchar ** in,
   }
 
   if (!GST_IS_PLUGIN_FEATURE (feature)) {
-    GST_ERROR ("typename : '%s' is not a plgin feature", type_name);
+    GST_ERROR ("typename : '%s' is not a plugin feature", type_name);
     goto fail;
   }
 
@@ -710,7 +895,7 @@ gst_registry_binary_load_feature (GstRegistry * registry, gchar ** in,
     GstBinaryElementFactory *ef;
     GstElementFactory *factory = GST_ELEMENT_FACTORY (feature);
 
-    align32 (*in);
+    align (*in);
     GST_LOG ("Reading/casting for GstBinaryElementFactory at address %p", *in);
     unpack_element (*in, ef, GstBinaryElementFactory);
     pf = (GstBinaryPluginFeature *) ef;
@@ -735,7 +920,7 @@ gst_registry_binary_load_feature (GstRegistry * registry, gchar ** in,
     if (ef->nuriprotocols) {
       GST_DEBUG ("Reading %d UriTypes at address %p", ef->nuriprotocols, *in);
 
-      align32 (*in);
+      align (*in);
       factory->uri_type = *((guint *) * in);
       *in += sizeof (factory->uri_type);
       //unpack_element(*in, &factory->uri_type, factory->uri_type);
@@ -757,7 +942,7 @@ gst_registry_binary_load_feature (GstRegistry * registry, gchar ** in,
     GstBinaryTypeFindFactory *tff;
     GstTypeFindFactory *factory = GST_TYPE_FIND_FACTORY (feature);
 
-    align32 (*in);
+    align (*in);
     GST_DEBUG ("Reading/casting for GstBinaryPluginFeature at address %p", *in);
     unpack_element (*in, tff, GstBinaryTypeFindFactory);
     pf = (GstBinaryPluginFeature *) tff;
@@ -776,19 +961,19 @@ gst_registry_binary_load_feature (GstRegistry * registry, gchar ** in,
         factory->extensions[i] = str;
       }
     }
-  }
-#ifndef GST_DISABLE_INDEX
-  else if (GST_IS_INDEX_FACTORY (feature)) {
+  } else if (GST_IS_INDEX_FACTORY (feature)) {
     GstIndexFactory *factory = GST_INDEX_FACTORY (feature);
 
-    align32 (*in);
+    align (*in);
     GST_DEBUG ("Reading/casting for GstBinaryPluginFeature at address %p", *in);
     unpack_element (*in, pf, GstBinaryPluginFeature);
 
     /* unpack index factory strings */
     unpack_string (*in, factory->longdesc);
+  } else {
+    GST_WARNING ("unhandled factory type : %s", G_OBJECT_TYPE_NAME (feature));
+    goto fail;
   }
-#endif
 
   feature->rank = pf->rank;
 
@@ -811,6 +996,57 @@ fail:
   return FALSE;
 }
 
+static gchar **
+gst_registry_binary_load_plugin_dep_strv (gchar ** in, guint n)
+{
+  gchar **arr;
+
+  if (n == 0)
+    return NULL;
+
+  arr = g_new0 (gchar *, n + 1);
+  while (n > 0) {
+    unpack_string (*in, arr[n - 1]);
+    --n;
+  }
+  return arr;
+}
+
+static gboolean
+gst_registry_binary_load_plugin_dep (GstPlugin * plugin, gchar ** in)
+{
+  GstPluginDep *dep;
+  GstBinaryDep *d;
+  gchar **s;
+
+  align (*in);
+  GST_LOG_OBJECT (plugin, "Unpacking GstBinaryDep from %p", *in);
+  unpack_element (*in, d, GstBinaryDep);
+
+  dep = g_new0 (GstPluginDep, 1);
+
+  dep->env_hash = d->env_hash;
+  dep->stat_hash = d->stat_hash;
+
+  dep->flags = d->flags;
+
+  dep->names = gst_registry_binary_load_plugin_dep_strv (in, d->n_names);
+  dep->paths = gst_registry_binary_load_plugin_dep_strv (in, d->n_paths);
+  dep->env_vars = gst_registry_binary_load_plugin_dep_strv (in, d->n_env_vars);
+
+  plugin->priv->deps = g_list_append (plugin->priv->deps, dep);
+
+  GST_DEBUG_OBJECT (plugin, "Loaded external plugin dependency from registry: "
+      "env_hash: %08x, stat_hash: %08x", dep->env_hash, dep->stat_hash);
+  for (s = dep->env_vars; s != NULL && *s != NULL; ++s)
+    GST_LOG_OBJECT (plugin, " evar: %s", *s);
+  for (s = dep->paths; s != NULL && *s != NULL; ++s)
+    GST_LOG_OBJECT (plugin, " path: %s", *s);
+  for (s = dep->names; s != NULL && *s != NULL; ++s)
+    GST_LOG_OBJECT (plugin, " name: %s", *s);
+
+  return TRUE;
+}
 
 /*
  * gst_registry_binary_load_plugin:
@@ -826,20 +1062,10 @@ gst_registry_binary_load_plugin (GstRegistry * registry, gchar ** in)
   GstPlugin *plugin = NULL;
   guint i;
 
-  align32 (*in);
+  align (*in);
   GST_LOG ("Reading/casting for GstBinaryPluginElement at address %p", *in);
   unpack_element (*in, pe, GstBinaryPluginElement);
 
-  if (pe->nfeatures < 0) {
-    GST_ERROR ("The number of feature structure is not valid !");
-    return FALSE;
-  }
-
-  if (pe->file_mtime < 0 || pe->file_size < 0) {
-    GST_ERROR ("Plugin time or file size is not valid !");
-    return FALSE;
-  }
-
   plugin = g_object_new (GST_TYPE_PLUGIN, NULL);
 
   /* TODO: also set GST_PLUGIN_FLAG_CONST */
@@ -856,12 +1082,23 @@ gst_registry_binary_load_plugin (GstRegistry * registry, gchar ** in)
   unpack_const_string (*in, plugin->desc.source);
   unpack_const_string (*in, plugin->desc.package);
   unpack_const_string (*in, plugin->desc.origin);
-  GST_LOG ("read strings for '%s'", plugin->desc.name);
+  GST_LOG ("read strings for name='%s'", plugin->desc.name);
+  GST_LOG ("  desc.description='%s'", plugin->desc.description);
+  GST_LOG ("  filename='%s'", plugin->filename);
+  GST_LOG ("  desc.version='%s'", plugin->desc.version);
+  GST_LOG ("  desc.license='%s'", plugin->desc.license);
+  GST_LOG ("  desc.source='%s'", plugin->desc.source);
+  GST_LOG ("  desc.package='%s'", plugin->desc.package);
+  GST_LOG ("  desc.origin='%s'", plugin->desc.origin);
 
   plugin->basename = g_path_get_basename (plugin->filename);
+
+  /* Takes ownership of plugin */
   gst_registry_add_plugin (registry, plugin);
-  GST_INFO ("Added plugin '%s' plugin with %d features from binary registry",
+  GST_DEBUG ("Added plugin '%s' plugin with %d features from binary registry",
       plugin->desc.name, pe->nfeatures);
+
+  /* Load plugin features */
   for (i = 0; i < pe->nfeatures; i++) {
     if (!gst_registry_binary_load_feature (registry, in, plugin->desc.name)) {
       GST_ERROR ("Error while loading binary feature");
@@ -869,11 +1106,18 @@ gst_registry_binary_load_plugin (GstRegistry * registry, gchar ** in)
     }
   }
 
+  /* Load external plugin dependencies */
+  for (i = 0; i < pe->n_deps; ++i) {
+    if (!gst_registry_binary_load_plugin_dep (plugin, in)) {
+      GST_ERROR_OBJECT (plugin, "Could not read external plugin dependency");
+      goto fail;
+    }
+  }
+
   return TRUE;
 
   /* Errors */
 fail:
-  gst_object_unref (plugin);
   return FALSE;
 }
 
@@ -891,22 +1135,25 @@ gboolean
 gst_registry_binary_read_cache (GstRegistry * registry, const char *location)
 {
   GMappedFile *mapped = NULL;
-  GTimer *timer = NULL;
   gchar *contents = NULL;
   gchar *in = NULL;
-  gdouble seconds;
   gsize size;
   GError *err = NULL;
   gboolean res = FALSE;
+  gint check_magic_result;
+#ifndef GST_DISABLE_GST_DEBUG
+  GTimer *timer = NULL;
+  gdouble seconds;
+#endif
 
   /* make sure these types exist */
   GST_TYPE_ELEMENT_FACTORY;
   GST_TYPE_TYPE_FIND_FACTORY;
-#ifndef GST_DISABLE_INDEX
   GST_TYPE_INDEX_FACTORY;
-#endif
 
+#ifndef GST_DISABLE_GST_DEBUG
   timer = g_timer_new ();
+#endif
 
   mapped = g_mapped_file_new (location, FALSE, &err);
   if (err != NULL) {
@@ -937,10 +1184,12 @@ gst_registry_binary_read_cache (GstRegistry * registry, const char *location)
     goto Error;
   }
   /* check if header is valid */
-  if (!gst_registry_binary_check_magic (&in)) {
-    GST_ERROR
-        ("Binary registry type not recognized (invalid magic) for file at %s",
-        location);
+  if ((check_magic_result = gst_registry_binary_check_magic (&in, size)) < 0) {
+
+    if (check_magic_result == -1)
+      GST_ERROR
+          ("Binary registry type not recognized (invalid magic) for file at %s",
+          location);
     goto Error;
   }
 
@@ -964,8 +1213,10 @@ gst_registry_binary_read_cache (GstRegistry * registry, const char *location)
     }
   }
 
+#ifndef GST_DISABLE_GST_DEBUG
   g_timer_stop (timer);
   seconds = g_timer_elapsed (timer, NULL);
+#endif
 
   GST_INFO ("loaded %s in %lf seconds", location, seconds);
 
@@ -973,7 +1224,9 @@ gst_registry_binary_read_cache (GstRegistry * registry, const char *location)
   /* TODO: once we re-use the pointers to registry contents return here */
 
 Error:
+#ifndef GST_DISABLE_GST_DEBUG
   g_timer_destroy (timer);
+#endif
   if (mapped) {
     g_mapped_file_free (mapped);
   } else {
