@@ -59,7 +59,7 @@ static gboolean
 gst_registry_save (GstRegistry * registry, gchar * format, ...)
 {
   va_list var_args;
-  size_t written, len;
+  gsize written, len;
   gboolean ret;
   char *str;
 
@@ -123,9 +123,15 @@ read_string (xmlTextReaderPtr reader, gchar ** write_to, gboolean allow_blank)
       return found;
     }
     if (xmlTextReaderNodeType (reader) == XML_READER_TYPE_TEXT) {
+      xmlChar *value;
+
       if (found)
         return FALSE;
-      *write_to = (gchar *) xmlTextReaderValue (reader);
+
+      value = xmlTextReaderValue (reader);
+      *write_to = g_strdup ((gchar *) value);
+      xmlFree (value);
+
       found = TRUE;
     }
   }
@@ -255,21 +261,20 @@ load_feature (xmlTextReaderPtr reader)
 {
   int ret;
   int depth;
-  gchar *feature_name;
+  xmlChar *feature_name;
   GstPluginFeature *feature;
   GType type;
 
   depth = xmlTextReaderDepth (reader);
-  feature_name =
-      (gchar *) xmlTextReaderGetAttribute (reader, BAD_CAST "typename");
+  feature_name = xmlTextReaderGetAttribute (reader, BAD_CAST "typename");
 
-  GST_LOG ("loading feature");
+  GST_LOG ("loading feature '%s'", GST_STR_NULL ((const char *) feature_name));
 
   if (!feature_name)
     return NULL;
 
-  type = g_type_from_name (feature_name);
-  g_free (feature_name);
+  type = g_type_from_name ((const char *) feature_name);
+  xmlFree (feature_name);
   feature_name = NULL;
 
   if (!type) {
@@ -365,13 +370,11 @@ load_feature (xmlTextReaderPtr reader)
             g_free (s);
           }
         }
-#ifndef GST_DISABLE_INDEX
       } else if (GST_IS_INDEX_FACTORY (feature)) {
         GstIndexFactory *factory = GST_INDEX_FACTORY (feature);
 
         if (g_str_equal (tag, "longdesc"))
           read_string (reader, &factory->longdesc, TRUE);
-#endif
       }
     }
   }
@@ -511,9 +514,7 @@ gst_registry_xml_read_cache (GstRegistry * registry, const char *location)
   /* make sure these types exist */
   GST_TYPE_ELEMENT_FACTORY;
   GST_TYPE_TYPE_FIND_FACTORY;
-#ifndef GST_DISABLE_INDEX
   GST_TYPE_INDEX_FACTORY;
-#endif
 
   timer = g_timer_new ();
 
@@ -769,12 +770,10 @@ gst_registry_xml_save_feature (GstRegistry * registry,
         i++;
       }
     }
-#ifndef GST_DISABLE_INDEX
   } else if (GST_IS_INDEX_FACTORY (feature)) {
     if (!gst_registry_save_escaped (registry, "  ", "longdesc",
             GST_INDEX_FACTORY (feature)->longdesc))
       return FALSE;
-#endif
   }
   return TRUE;
 }
@@ -786,6 +785,10 @@ gst_registry_xml_save_plugin (GstRegistry * registry, GstPlugin * plugin)
   GList *walk;
   char s[100];
 
+  if (plugin->priv->deps != NULL) {
+    GST_WARNING ("XML registry does not support external plugin dependencies");
+  }
+
   if (!gst_registry_save_escaped (registry, " ", "name", plugin->desc.name))
     return FALSE;
   if (!gst_registry_save_escaped (registry, " ", "description",
@@ -922,7 +925,7 @@ gst_registry_xml_write_cache (GstRegistry * registry, const char *location)
 
   if (g_file_test (tmp_location, G_FILE_TEST_EXISTS)) {
 #ifdef WIN32
-    g_remove (location);
+    g_unlink (location);
 #endif
     if (g_rename (tmp_location, location) < 0)
       goto rename_failed;
@@ -942,7 +945,7 @@ fail:
   }
 fail_after_close:
   {
-    g_remove (tmp_location);
+    g_unlink (tmp_location);
     g_free (tmp_location);
     return FALSE;
   }
