@@ -285,7 +285,7 @@ unescape_string (const gchar * escaped_string, const gchar * illegal_characters)
   }
 
   *out = '\0';
-  g_assert ((size_t) (out - result) <= strlen (escaped_string));
+  g_assert ((gsize) (out - result) <= strlen (escaped_string));
   return result;
 
 }
@@ -301,7 +301,8 @@ gst_uri_protocol_check_internal (const gchar * uri, gchar ** endptr)
 
   if (g_ascii_isalpha (*check)) {
     check++;
-    while (g_ascii_isalnum (*check))
+    while (g_ascii_isalnum (*check) || *check == '+'
+        || *check == '-' || *check == '.')
       check++;
   }
 
@@ -313,7 +314,8 @@ gst_uri_protocol_check_internal (const gchar * uri, gchar ** endptr)
  * @protocol: A string
  *
  * Tests if the given string is a valid protocol identifier. Protocols
- * must consist of alphanumeric characters and not start with a number.
+ * must consist of alphanumeric characters, '+', '-' and '.' and must
+ * start with a alphabetic character. See RFC 3986 Section 3.1.
  *
  * Returns: TRUE if the string is a valid protocol identifier, FALSE otherwise.
  */
@@ -334,7 +336,7 @@ gst_uri_protocol_is_valid (const gchar * protocol)
  * @uri: A URI string
  *
  * Tests if the given string is a valid URI identifier. URIs start with a valid
- * protocol followed by "://" and maybe a string identifying the location.
+ * scheme followed by ":" and maybe a string identifying the location.
  *
  * Returns: TRUE if the string is a valid URI
  */
@@ -347,7 +349,7 @@ gst_uri_is_valid (const gchar * uri)
 
   gst_uri_protocol_check_internal (uri, &endptr);
 
-  return (*endptr == ':' && *(endptr + 1) == '/' && *(endptr + 2) == '/');
+  return *endptr == ':';
 }
 
 /**
@@ -367,9 +369,9 @@ gst_uri_get_protocol (const gchar * uri)
   g_return_val_if_fail (uri != NULL, NULL);
   g_return_val_if_fail (gst_uri_is_valid (uri), NULL);
 
-  colon = strstr (uri, "://");
+  colon = strstr (uri, ":");
 
-  return g_strdown (g_strndup (uri, colon - uri));
+  return g_ascii_strdown (uri, colon - uri);
 }
 
 /**
@@ -392,40 +394,57 @@ gst_uri_has_protocol (const gchar * uri, const gchar * protocol)
   g_return_val_if_fail (protocol != NULL, FALSE);
   g_return_val_if_fail (gst_uri_is_valid (uri), FALSE);
 
-  colon = strstr (uri, "://");
+  colon = strstr (uri, ":");
 
   if (colon == NULL)
     return FALSE;
 
-  return (strncmp (uri, protocol, (size_t) (colon - uri)) == 0);
+  return (g_ascii_strncasecmp (uri, protocol, (gsize) (colon - uri)) == 0);
 }
 
 /**
  * gst_uri_get_location:
  * @uri: A URI string
  *
- * Extracts the location out of a given valid URI. So the protocol and "://"
- * are stripped from the URI. The returned string must be freed using
+ * Extracts the location out of a given valid URI, ie. the protocol and "://"
+ * are stripped from the URI, which means that the location returned includes
+ * the hostname if one is specified. The returned string must be freed using
  * g_free().
  *
- * Returns: The location for this URI. Returns NULL if the URI isn't valid.
+ * Returns: The location for this URI. Returns NULL if the URI isn't valid. If
+ * the URI does not contain a location, an empty string is returned.
  */
 gchar *
 gst_uri_get_location (const gchar * uri)
 {
-  gchar *colon;
-  gchar *location, *unescaped;
+  const gchar *colon;
+  gchar *unescaped = NULL;
 
   g_return_val_if_fail (uri != NULL, NULL);
   g_return_val_if_fail (gst_uri_is_valid (uri), NULL);
 
   colon = strstr (uri, "://");
+  if (!colon)
+    return NULL;
 
-  location = g_strdup (colon + 3);
+  unescaped = unescape_string (colon + 3, "/");
 
-  unescaped = unescape_string (location, "/");
-  g_free (location);
+  /* On Windows an URI might look like file:///c:/foo/bar.txt or
+   * file:///c|/foo/bar.txt (some Netscape versions) and we want to
+   * return c:/foo/bar.txt as location rather than /c:/foo/bar.txt.
+   * Can't use g_filename_from_uri() here because it will only handle the
+   * file:// protocol */
+#ifdef G_OS_WIN32
+  if (unescaped != NULL && unescaped[0] == '/' &&
+      g_ascii_isalpha (unescaped[1]) &&
+      (unescaped[2] == ':' || unescaped[2] == '|')) {
+    unescaped[2] = ':';
+    g_memmove (unescaped, unescaped + 1, strlen (unescaped + 1) + 1);
+  }
+#endif
 
+  GST_LOG ("extracted location '%s' from URI '%s'", GST_STR_NULL (unescaped),
+      uri);;
   return unescaped;
 }
 
@@ -442,15 +461,17 @@ gst_uri_get_location (const gchar * uri)
 gchar *
 gst_uri_construct (const gchar * protocol, const gchar * location)
 {
-  char *escaped;
+  char *escaped, *proto_lowercase;
   char *retval;
 
   g_return_val_if_fail (gst_uri_protocol_is_valid (protocol), NULL);
   g_return_val_if_fail (location != NULL, NULL);
 
+  proto_lowercase = g_ascii_strdown (protocol, -1);
   escaped = escape_string (location);
-  retval = g_strdup_printf ("%s://%s", protocol, escaped);
+  retval = g_strdup_printf ("%s://%s", proto_lowercase, escaped);
   g_free (escaped);
+  g_free (proto_lowercase);
 
   return retval;
 }
@@ -647,8 +668,14 @@ gst_uri_handler_get_protocols (GstURIHandler * handler)
 
   iface = GST_URI_HANDLER_GET_INTERFACE (handler);
   g_return_val_if_fail (iface != NULL, NULL);
-  g_return_val_if_fail (iface->get_protocols != NULL, NULL);
-  ret = iface->get_protocols ();
+  g_return_val_if_fail (iface->get_protocols != NULL ||
+      iface->get_protocols_full != NULL, NULL);
+
+  if (iface->get_protocols != NULL) {
+    ret = iface->get_protocols ();
+  } else {
+    ret = iface->get_protocols_full (G_OBJECT_TYPE (handler));
+  }
   g_return_val_if_fail (ret != NULL, NULL);
 
   return ret;
@@ -695,6 +722,8 @@ gboolean
 gst_uri_handler_set_uri (GstURIHandler * handler, const gchar * uri)
 {
   GstURIHandlerInterface *iface;
+  gboolean ret;
+  gchar *new_uri, *protocol, *location, *colon;
 
   g_return_val_if_fail (GST_IS_URI_HANDLER (handler), FALSE);
   g_return_val_if_fail (gst_uri_is_valid (uri), FALSE);
@@ -702,7 +731,21 @@ gst_uri_handler_set_uri (GstURIHandler * handler, const gchar * uri)
   iface = GST_URI_HANDLER_GET_INTERFACE (handler);
   g_return_val_if_fail (iface != NULL, FALSE);
   g_return_val_if_fail (iface->set_uri != NULL, FALSE);
-  return iface->set_uri (handler, uri);
+
+  protocol = gst_uri_get_protocol (uri);
+
+  colon = strstr (uri, ":");
+  location = g_strdup (colon);
+
+  new_uri = g_strdup_printf ("%s%s", protocol, location);
+
+  ret = iface->set_uri (handler, uri);
+
+  g_free (new_uri);
+  g_free (location);
+  g_free (protocol);
+
+  return ret;
 }
 
 /**
