@@ -22,6 +22,7 @@
 
 #include <gst/gstconfig.h>
 #include <gst/gstcaps.h>
+#include <gst/gststructure.h>
 
 G_BEGIN_DECLS
 
@@ -140,6 +141,16 @@ G_BEGIN_DECLS
  */
 #define GST_VALUE_HOLDS_CAPS(x)		(G_VALUE_HOLDS(x, GST_TYPE_CAPS))
 
+/**
+ * GST_VALUE_HOLDS_STRUCTURE:
+ * @x: the #GValue to check
+ *
+ * Checks if the given #GValue contains a #GST_TYPE_STRUCTURE value.
+ *
+ * Since: 0.10.15
+ */
+#define GST_VALUE_HOLDS_STRUCTURE(x)		(G_VALUE_HOLDS(x, GST_TYPE_STRUCTURE))
+
 /**
  * GST_VALUE_HOLDS_BUFFER:
  * @x: the #GValue to check
@@ -203,7 +214,11 @@ G_BEGIN_DECLS
 /**
  * GST_TYPE_LIST:
  *
- * a #GValue type that represents an unordered list of #GValue values
+ * a #GValue type that represents an unordered list of #GValue values. This
+ * is used for example to express a list of possible values for a field in
+ * a caps structure, like a list of possible sample rates, of which only one
+ * will be chosen in the end. This means that all values in the list are
+ * meaningful on their own.
  *
  * Returns: the #GType of GstValueList (which is not explicitly typed)
  */
@@ -212,7 +227,12 @@ G_BEGIN_DECLS
 /**
  * GST_TYPE_ARRAY:
  *
- * a #GValue type that represents an ordered list of #GValue values
+ * a #GValue type that represents an ordered list of #GValue values. This is
+ * used to express a set of values that is meaningful only in their specific
+ * combination and order of values. Each value on its own is not particularly
+ * meaningful, only the ordered array in its entirety is meaningful. This is
+ * used for example to express channel layouts for multichannel audio where
+ * each channel needs to be mapped to a position in the room.
  *
  * Returns: the #GType of GstArrayList (which is not explicitly typed)
  */
@@ -440,6 +460,12 @@ G_CONST_RETURN GstCaps *
 void		gst_value_set_caps		(GValue		*value,
 						 const GstCaps  *caps);
 
+/* structure */
+G_CONST_RETURN GstStructure *
+		gst_value_get_structure		(const GValue	*value);
+void		gst_value_set_structure		(GValue		*value,
+						 const GstStructure  *structure);
+
 /* fraction */
 void		gst_value_set_fraction		(GValue		*value,
 						 gint		numerator,
@@ -476,7 +502,6 @@ gint		gst_value_compare		(const GValue	*value1,
 						 const GValue	*value2);
 gboolean	gst_value_can_compare		(const GValue	*value1,
 						 const GValue	*value2);
-
 /* union */
 gboolean	gst_value_union			(GValue		*dest,
 						 const GValue	*value1,
@@ -510,9 +535,6 @@ void		gst_value_register_subtract_func (GType		minuend_type,
 /* fixation */
 gboolean	gst_value_is_fixed		(const GValue   *value);
 
-/* private */
-void		_gst_value_initialize		(void);
-
 G_END_DECLS
 
 #endif
