@@ -21,9 +21,8 @@
  */
 /**
  * SECTION:element-capsfilter
- * @short_description: pass data though unmodified, but enforces format limmits
  *
- * The element does not modify data as such, but can enforce limmitations on the
+ * The element does not modify data as such, but can enforce limitations on the
  * data format.
  */
 
@@ -68,6 +67,7 @@ static void gst_capsfilter_set_property (GObject * object, guint prop_id,
 static void gst_capsfilter_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 static void gst_capsfilter_dispose (GObject * object);
+
 static GstCaps *gst_capsfilter_transform_caps (GstBaseTransform * base,
     GstPadDirection direction, GstCaps * caps);
 static GstFlowReturn gst_capsfilter_transform_ip (GstBaseTransform * base,
@@ -125,6 +125,14 @@ gst_capsfilter_init (GstCapsFilter * filter, GstCapsFilterClass * g_class)
   filter->filter_caps = gst_caps_new_any ();
 }
 
+static gboolean
+copy_func (GQuark field_id, const GValue * value, GstStructure * dest)
+{
+  gst_structure_id_set_value (dest, field_id, value);
+
+  return TRUE;
+}
+
 static void
 gst_capsfilter_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -134,7 +142,7 @@ gst_capsfilter_set_property (GObject * object, guint prop_id,
   switch (prop_id) {
     case PROP_FILTER_CAPS:{
       GstCaps *new_caps;
-      GstCaps *old_caps;
+      GstCaps *old_caps, *suggest, *nego;
       const GstCaps *new_caps_val = gst_value_get_caps (value);
 
       if (new_caps_val == NULL) {
@@ -144,12 +152,53 @@ gst_capsfilter_set_property (GObject * object, guint prop_id,
         gst_caps_ref (new_caps);
       }
 
+      GST_OBJECT_LOCK (capsfilter);
       old_caps = capsfilter->filter_caps;
       capsfilter->filter_caps = new_caps;
+      GST_OBJECT_UNLOCK (capsfilter);
+
       gst_caps_unref (old_caps);
 
       GST_DEBUG_OBJECT (capsfilter, "set new caps %" GST_PTR_FORMAT, new_caps);
 
+      /* filter the currently negotiated format against the new caps */
+      GST_OBJECT_LOCK (GST_BASE_TRANSFORM_SINK_PAD (object));
+      nego = GST_PAD_CAPS (GST_BASE_TRANSFORM_SINK_PAD (object));
+      if (nego) {
+        GstStructure *s1, *s2;
+
+        /* first check if the name is the same */
+        s1 = gst_caps_get_structure (nego, 0);
+        s2 = gst_caps_get_structure (new_caps, 0);
+
+        GST_DEBUG_OBJECT (capsfilter, "we had negotiated caps %" GST_PTR_FORMAT,
+            nego);
+
+        if (gst_structure_get_name_id (s1) == gst_structure_get_name_id (s2)) {
+          /* same name, copy all fields from the new caps into the previously
+           * negotiated caps */
+          suggest = gst_caps_copy (nego);
+          s1 = gst_caps_get_structure (suggest, 0);
+          gst_structure_foreach (s2, (GstStructureForeachFunc) copy_func, s1);
+          GST_DEBUG_OBJECT (capsfilter, "copied structure fields");
+        } else {
+          GST_DEBUG_OBJECT (capsfilter, "different structure names");
+          /* different names, we can only suggest the complete caps */
+          suggest = gst_caps_copy (new_caps);
+        }
+      } else {
+        GST_DEBUG_OBJECT (capsfilter, "no negotiated caps");
+        /* no previous caps, the getcaps function will be used to find suitable
+         * caps */
+        suggest = NULL;
+      }
+      GST_OBJECT_UNLOCK (GST_BASE_TRANSFORM_SINK_PAD (object));
+
+      if (suggest) {
+        gst_base_transform_suggest (GST_BASE_TRANSFORM (object), suggest, 0);
+        gst_caps_unref (suggest);
+      }
+
       /* FIXME: Need to activate these caps on the pads
        * http://bugzilla.gnome.org/show_bug.cgi?id=361718
        */
@@ -169,7 +218,9 @@ gst_capsfilter_get_property (GObject * object, guint prop_id, GValue * value,
 
   switch (prop_id) {
     case PROP_FILTER_CAPS:
+      GST_OBJECT_LOCK (capsfilter);
       gst_value_set_caps (value, capsfilter->filter_caps);
+      GST_OBJECT_UNLOCK (capsfilter);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -192,9 +243,17 @@ gst_capsfilter_transform_caps (GstBaseTransform * base,
     GstPadDirection direction, GstCaps * caps)
 {
   GstCapsFilter *capsfilter = GST_CAPSFILTER (base);
-  GstCaps *ret;
+  GstCaps *ret, *filter_caps;
 
-  ret = gst_caps_intersect (caps, capsfilter->filter_caps);
+  GST_OBJECT_LOCK (capsfilter);
+  filter_caps = gst_caps_ref (capsfilter->filter_caps);
+  GST_OBJECT_UNLOCK (capsfilter);
+
+  ret = gst_caps_intersect (caps, filter_caps);
+  GST_DEBUG_OBJECT (capsfilter, "filter:    %" GST_PTR_FORMAT, filter_caps);
+  GST_DEBUG_OBJECT (capsfilter, "intersect: %" GST_PTR_FORMAT, ret);
+
+  gst_caps_unref (filter_caps);
 
   return ret;
 }
@@ -232,6 +291,8 @@ static GstFlowReturn
 gst_capsfilter_prepare_buf (GstBaseTransform * trans, GstBuffer * input,
     gint size, GstCaps * caps, GstBuffer ** buf)
 {
+  GstFlowReturn ret = GST_FLOW_OK;
+
   if (GST_BUFFER_CAPS (input) != NULL) {
     /* Output buffer already has caps */
     GST_DEBUG_OBJECT (trans,
@@ -273,11 +334,19 @@ gst_capsfilter_prepare_buf (GstBaseTransform * trans, GstBuffer * input,
       if (GST_PAD_CAPS (trans->srcpad) == NULL)
         gst_pad_set_caps (trans->srcpad, out_caps);
     } else {
-      GST_DEBUG_OBJECT (trans, "Have unfixed output caps %" GST_PTR_FORMAT,
-          out_caps);
+      gchar *caps_str = gst_caps_to_string (out_caps);
+
+      GST_DEBUG_OBJECT (trans, "Cannot choose caps. Have unfixed output caps %"
+          GST_PTR_FORMAT, out_caps);
       gst_caps_unref (out_caps);
+
+      ret = GST_FLOW_ERROR;
+      GST_ELEMENT_ERROR (trans, STREAM, FORMAT,
+          ("Filter caps do not completely specify the output format"),
+          ("Output caps are unfixed: %s", caps_str));
+      g_free (caps_str);
     }
   }
 
-  return GST_FLOW_OK;
+  return ret;
 }
